/**
 *
 * Jquery Mapael - Dynamic maps jQuery plugin (based on raphael.js)
 * Requires jQuery and raphael.js
 *
 * Map of Canada
 * Lambert Conformal Conic Projection
 * 2016-04-06
 * @author Ryan Jarvis (Cabalist)
 * @source Wikicommons
 */
(function (factory) {
    if (typeof exports === 'object') {
        // CommonJS
        module.exports = factory(require('jquery'), require('jquery-mapael'));
    } else if (typeof define === 'function' && define.amd) {
        // AMD. Register as an anonymous module.
        define(['jquery', 'mapael'], factory);
    } else {
        // Browser globals
        factory(jQuery, jQuery.mapael);
    }
}(function ($, Mapael) {

    "use strict";

    var EPSLN = 1.0e-10;
    var SPI = 3.14159265359;
    var D2R = 0.01745329251994329577;
    var TWO_PI = Math.PI * 2;
    var HALF_PI = Math.PI / 2;

    var long0 = -1.6755160819145565;
    var a = 6378137;
    var e = 0.08181919104281582;
    var rh = 5850046.500911157;
    var ns = 0.8706189996062497;
    var f0 = 1.7754029480833013;

    var tsfnz = function (eccent, phi, sinphi) {
        var con = eccent * sinphi;
        var com = 0.5 * eccent;
        con = Math.pow(((1 - con) / (1 + con)), com);
        return (Math.tan(0.5 * (HALF_PI - phi)) / con);
    };

    var sign = function (x) {
        return x < 0 ? -1 : 1;
    };

    var adjust_lon = function (x) {
        return (Math.abs(x) <= SPI) ? x : (x - (sign(x) * TWO_PI));
    };

    var getGeoXY = function (lon, lat) {
        lon *= D2R;
        lat *= D2R;

        if (Math.abs(2 * Math.abs(lat) - Math.PI) <= EPSLN) {
            lat = sign(lat) * (HALF_PI - 2 * EPSLN);
        }

        var con = Math.abs(Math.abs(lat) - HALF_PI);
        var ts, rh1;
        if (con > EPSLN) {
            ts = tsfnz(e, lat, Math.sin(lat));
            rh1 = a * f0 * Math.pow(ts, ns);
        }
        else {
            con = lat * ns;
            if (con <= 0) {
                return {x: NaN, y: NaN};
            }
            rh1 = 0;
        }
        var theta = ns * adjust_lon(lon - long0);
        
        return {
            x: rh1 * Math.sin(theta),
            y: rh - rh1 * Math.cos(theta)
        };
    };

    var svgWidth = 1064;
    var svgHeight = 1003;

    // Geo extent of viewport
    var xMin = -2510795.26;
    var xMax = 3229899.87;
    var yMin = 24835.61;
    var yMax = 5436102.32;
    var xSize = xMax - xMin;
    var ySize = yMax - yMin;
    var xRatio = svgWidth / xSize;
    var yRatio = svgHeight / ySize;

    $.extend(true, Mapael,
        {
            maps: {
                canada: {
                    width: svgWidth,
                    height: svgHeight,
                    getCoords: function (lat, lon) {
                        // Get projected coords
                        var coords = getGeoXY(lon, lat);

                        // Adjust to viewport
                        coords.x -= xMin;
                        coords.y = yMax - coords.y;
                        coords.x *= xRatio;
                        coords.y *= yRatio;

                        return coords;
                    },
                    elems: {
                        "QC": "m 935,624 -1,1 -1,-1 -3,3 -1,0 0,1 -1,-1 0,1 0,1 -1,0 -1,3 -1,3 -1,0 0,-1 -1,0 1,1 -2,0 -1,3 -1,-1 1,1 -1,0 2,1 -2,3 1,1 -1,1 0,-1 -1,1 1,0 1,2 0,1 1,0 -1,2 1,0 -1,0 -1,-1 0,-1 -1,0 1,2 0,2 -1,0 0,1 0,1 -1,0 1,1 -1,0 0,4 -2,2 0,2 1,0 0,2 -2,2 -1,0 0,-1 0,2 -1,0 0,-1 0,1 -1,0 0,1 -1,0 -1,1 -1,0 0,1 -1,0 0,1 -1,0 0,-1 -1,2 -1,0 -1,1 0,1 -2,0 -4,5 -1,0 2,-2 0,-1 -2,3 -1,-1 -5,2 -3,0 0,1 -3,1 0,-1 -1,2 -2,1 -1,-1 -1,2 -2,2 0,1 -2,-1 -1,1 -3,0 -4,3 -4,1 -4,4 -1,0 -3,1 -1,0 -5,4 -3,1 -1,2 0,-1 0,1 -2,1 -1,1 -2,-1 0,2 1,0 0,1 -2,0 0,3 -2,2 0,1 0,3 0,-1 -1,2 0,1 1,1 0,1 0,1 1,2 -1,4 -2,1 -2,2 -2,1 -2,1 0,2 -2,0 2,0 0,1 -1,1 -1,2 0,-2 -2,-1 2,1 -2,3 0,2 -1,0 1,1 -1,1 -2,2 0,1 -1,1 1,3 -1,1 0,3 -1,3 -1,3 0,2 -2,0 -3,-1 -3,1 -3,-1 -4,2 -2,-1 -3,1 -1,1 2,-1 2,0 -1,1 1,1 1,-1 4,-2 3,0 1,1 4,-1 1,1 2,0 0,7 -1,4 -1,2 1,1 -3,3 0,3 -1,4 -2,4 -2,4 -1,2 2,-2 1,-1 5,-6 2,-4 0,-3 2,-3 -1,-2 2,-1 2,-8 -1,0 1,-3 1,-1 4,-10 1,-1 1,-2 1,-2 3,-5 1,-1 4,-5 5,-5 4,-6 5,-4 5,-3 2,-1 8,-4 5,0 6,2 3,2 -3,0 -3,0 1,1 1,-1 1,1 -1,0 2,0 3,1 -1,1 0,1 2,0 -1,2 0,1 -4,4 1,1 0,2 -3,2 1,0 -3,5 -2,1 -1,-1 -4,1 -2,0 0,-1 -2,3 -2,1 0,1 -3,0 0,2 -3,3 -1,1 0,1 -1,0 -2,2 -1,0 0,2 -4,-1 -1,-1 -6,3 0,1 -3,2 2,5 1,2 -1,3 -5,6 -1,-3 -3,1 -4,18 1,3 1,3 -2,2 0,4 1,0 0,2 1,2 1,0 0,1 -1,3 0,1 -1,2 -1,4 2,2 -3,0 0,1 1,1 0,1 0,2 -2,-2 -2,1 0,2 -2,0 -1,1 -1,0 1,1 0,2 -1,1 1,1 -6,2 -9,4 -9,3 -2,1 -9,3 -10,4 3,-3 0,-2 1,-1 -3,-2 0,-5 -4,0 -3,1 -8,5 -4,4 -4,-1 -2,0 -2,2 -4,-1 -1,0 -1,-3 -2,0 -1,-2 -1,0 0,1 -1,1 -3,-1 -2,-1 -1,-1 -4,-2 -2,0 -15,1 -1,-1 -3,-2 -5,-5 -2,-2 -3,-3 -1,-3 -3,-3 1,-3 -6,-21 -1,-3 -11,-44 3,3 0,1 -1,-2 0,-1 -2,-1 -1,-4 -2,-6 1,0 -1,-2 2,-2 1,0 0,1 0,1 2,-1 2,1 2,4 0,1 1,1 -1,-1 0,-2 1,-1 -1,-1 1,-2 1,0 -2,0 0,-2 -2,-1 -1,0 -1,-2 2,-2 -1,-1 1,-1 1,-1 -1,-1 1,0 0,-1 1,-1 0,-1 -1,-1 2,-1 -1,0 -1,0 0,-2 -1,-1 1,-1 -1,0 0,-1 -3,0 1,-1 -1,0 0,-1 -1,0 0,-1 -1,-1 1,-1 -1,0 1,-2 -2,1 -1,-1 1,-1 1,0 -1,0 0,-1 -2,1 1,-1 -1,0 0,-3 0,-1 0,-1 -1,0 0,-2 -1,-1 -1,1 0,-1 -1,-1 1,0 1,0 -1,-1 1,0 -1,-1 -1,0 0,-1 -2,0 2,-1 -1,-1 0,-1 2,0 -2,-1 -1,-1 1,0 -2,0 2,-1 -1,0 1,-1 -1,0 -1,-1 -2,1 1,-1 0,-1 1,1 -1,-1 1,0 -1,0 1,-1 -2,1 -2,0 1,-2 -1,0 -1,0 -1,-2 0,-1 -1,-1 -1,-1 1,-1 -1,0 -1,1 0,-2 0,1 0,-1 -1,1 1,-2 4,-3 3,-1 0,-2 6,-4 5,-7 1,-2 3,-6 1,-2 -1,0 2,-5 0,-2 2,-2 -1,-2 0,-3 0,-2 -2,-8 -2,-4 0,-1 -3,-6 -3,-4 -6,-6 -4,-2 0,-1 -2,-1 -5,-1 -1,-1 -2,-1 -2,-1 1,1 -1,0 -1,-1 -1,0 0,-1 1,-1 -2,-1 1,0 0,-1 -1,-2 -1,-1 2,1 0,-1 1,1 0,-2 0,-1 0,-1 1,0 0,-1 -1,0 1,-1 0,-1 2,-2 -1,0 1,-1 0,-1 1,0 -1,-1 -2,1 1,-1 -1,-1 2,-1 -1,-2 -1,-1 1,0 -1,0 1,0 1,-1 2,1 0,1 1,0 -1,0 0,-1 -1,-1 -1,-1 1,0 0,-1 1,0 -1,-2 -1,0 1,-2 1,0 -1,-1 1,-1 1,0 1,-1 1,-1 1,1 -1,-1 0,-1 -1,1 -3,0 1,1 -1,0 0,1 0,1 -1,0 -2,0 0,-1 1,-1 -1,-1 -1,-1 -1,0 0,-1 -1,-1 -1,0 2,-2 0,-1 1,-1 -2,0 -1,1 -2,-1 2,-3 -1,0 1,-2 -3,3 0,-1 -1,0 1,-1 -3,2 0,-2 1,-3 1,-2 0,-2 1,-1 -1,0 0,-1 -2,-3 1,0 0,-1 1,0 0,-1 -1,0 0,-1 2,0 -2,0 0,-1 -1,0 -1,-1 -1,0 -1,0 -1,-1 -1,-2 0,-1 -1,-1 -1,-3 -1,-3 1,-2 2,-3 2,-2 0,-1 1,1 3,0 1,-1 8,1 1,1 1,-1 3,0 1,1 -2,2 0,1 0,-1 2,-3 2,-1 2,0 0,1 0,-1 2,0 2,2 2,0 -2,-1 0,-2 2,-1 1,-1 2,-2 1,-2 1,-1 2,1 2,0 2,0 0,1 2,1 2,1 -1,0 2,0 -1,0 1,-1 1,0 1,1 -1,1 1,0 1,1 -1,1 0,1 0,1 1,-1 0,1 1,0 -1,-2 4,0 1,1 -1,0 1,1 1,0 1,1 0,1 -2,0 0,2 1,-1 1,0 1,-1 0,-1 -1,-1 3,0 1,0 1,1 -3,2 1,0 1,1 -1,1 3,-1 -1,1 -1,1 2,0 -1,1 2,0 0,1 1,1 0,-1 0,-1 2,2 0,-1 1,-1 1,1 1,0 1,0 1,0 1,0 0,-1 1,1 1,0 0,-1 1,0 0,-1 1,0 1,0 0,-1 1,0 0,1 0,1 1,0 0,1 0,1 1,0 1,0 1,-1 -1,1 1,-2 1,0 -1,-1 0,-2 0,-1 1,0 2,2 1,0 0,1 1,1 -3,4 1,1 1,1 -1,1 1,1 0,1 1,1 0,1 2,1 0,1 1,0 0,2 -1,1 -1,0 0,1 1,0 -5,2 -2,1 -5,1 5,0 3,-2 2,0 3,-2 0,1 0,1 2,0 0,1 1,0 0,2 1,0 0,2 0,1 -1,1 0,1 1,-1 1,1 0,1 1,0 -1,2 1,0 2,-2 1,0 1,0 0,1 0,1 -1,0 0,1 0,-1 -1,0 0,2 1,1 0,-1 2,0 -1,2 -1,-1 2,3 0,1 0,1 -1,1 -1,-1 0,-1 -1,0 0,-1 -1,0 -1,-1 0,1 0,1 1,0 1,2 -1,0 -3,3 3,0 0,-1 1,1 1,-1 0,2 1,1 0,-3 1,0 0,-1 1,-1 0,-3 2,-1 3,-1 1,-1 4,0 1,1 0,1 1,1 0,1 1,0 0,1 1,0 1,2 0,4 1,2 0,2 -5,4 -1,2 -1,3 3,-5 4,-4 1,-2 -1,-2 0,-1 -1,-3 0,-2 1,-1 0,-1 1,1 1,2 1,1 0,2 -1,3 1,1 0,-2 0,-1 1,-1 0,-2 -1,0 0,-3 1,0 0,1 1,1 -1,1 1,0 1,1 0,1 1,2 0,2 1,1 0,-1 0,-2 -1,-1 0,-4 1,0 1,-3 1,0 0,-1 1,-1 0,-1 0,1 1,0 0,-2 2,0 -1,-1 0,-1 0,-1 -1,-1 1,0 -1,-1 1,0 0,-1 -1,-1 1,-1 4,1 1,1 1,0 0,3 2,1 -1,1 1,1 -1,-1 1,-1 0,-1 -1,0 0,-2 0,-1 1,0 -2,0 -1,-1 -1,-1 -1,0 0,-2 1,0 1,-1 -2,0 1,-1 -1,-1 3,0 -1,-1 -1,0 1,-1 2,0 -1,-1 2,-1 -2,0 -1,1 -2,-1 0,-1 -1,-1 1,0 1,1 0,-1 1,-1 -2,0 0,-2 3,1 -2,-1 0,-1 -1,0 -1,-1 4,-1 1,1 1,0 -3,-1 -2,0 -3,-3 0,-1 1,-1 -1,-1 1,0 2,0 2,0 -3,0 -1,-1 1,0 -1,-1 1,0 -1,-1 1,0 -1,0 1,-1 -1,0 0,-1 0,-1 0,-1 0,-1 1,-1 -2,0 1,-1 -1,-2 2,0 2,0 -3,2 1,1 1,0 1,0 1,1 -1,0 0,1 0,1 -1,0 1,1 2,0 0,1 0,1 0,1 1,2 1,3 2,1 2,-3 1,1 -1,1 1,1 -1,1 1,0 0,1 1,1 0,1 1,0 0,1 2,0 1,1 -1,1 -3,0 -1,0 0,1 0,1 0,1 1,0 1,-1 2,1 2,-2 1,1 1,1 1,0 1,-2 1,-1 0,-1 2,0 2,1 -1,1 -4,3 1,1 0,1 1,1 1,-2 2,1 -2,4 0,3 -1,1 0,2 3,0 0,1 4,3 0,-1 1,0 1,1 1,0 0,-1 1,1 0,-1 1,-1 0,2 0,1 0,1 1,2 1,2 -1,1 1,2 0,-1 1,0 0,2 1,1 -1,1 1,1 0,2 1,0 -1,1 1,2 -1,0 0,2 4,2 0,1 1,0 0,1 -2,1 1,1 0,1 1,1 1,-1 1,0 0,2 0,1 2,-1 0,1 1,0 4,-2 -2,3 1,2 3,1 2,2 2,0 2,0 0,2 2,0 -1,2 -2,4 1,2 2,1 1,8 -3,1 -5,0 -1,0 -1,2 -3,1 -3,2 -4,1 -2,2 -3,-1 -1,-2 -1,0 -1,1 -2,0 -6,-3 -1,0 -1,-1 -1,0 -1,1 3,1 0,1 0,1 2,2 0,1 1,1 1,1 -1,1 1,1 -3,0 -2,-2 -4,0 0,-1 -2,0 0,1 3,3 1,0 0,1 2,1 0,1 2,0 -2,1 0,2 -2,0 -1,0 0,2 1,1 0,1 2,1 0,1 1,0 -1,2 -1,3 1,1 2,0 1,2 2,0 1,4 5,-1 0,1 1,1 1,1 0,-1 2,1 0,3 1,1 1,2 -1,2 2,2 0,2 1,0 2,0 -1,-2 2,2 1,-2 -2,-4 2,-1 -1,-2 2,0 2,2 -1,1 2,2 -1,1 1,2 1,0 -1,1 2,2 -1,1 1,1 1,2 2,1 0,-2 -1,-1 0,-1 2,1 1,0 1,2 2,0 2,0 -1,-1 1,-1 2,0 0,-1 0,-2 4,-2 5,2 1,1 4,1 -1,2 1,1 1,0 1,1 1,-2 1,-3 -2,-1 0,-1 -1,-1 0,-1 0,-1 -1,-2 2,-1 -2,-2 0,-2 -3,-3 0,-1 -1,-1 0,-2 -1,0 -1,-1 1,-2 2,0 -1,-1 0,-1 -1,-2 2,-2 1,0 1,1 1,1 2,4 2,0 1,1 -5,3 -1,1 0,3 1,0 1,0 1,1 -1,0 2,2 4,1 1,1 -2,1 1,0 70,-41 2,-2 7,10 m -121,167 -1,2 -2,2 2,-4 0,-1 1,1 m 113,-70 1,0 -1,2 -1,0 1,-1 -1,0 -1,3 -1,3 1,-1 0,-2 1,-1 -1,2 0,2 0,1 -1,1 0,1 1,0 1,0 1,-1 0,1 -1,1 -2,1 -1,-4 3,-7 1,-1 m -47,-26 8,-2 6,0 3,1 2,0 1,0 4,-1 5,3 0,1 -1,1 -5,3 -3,0 -9,3 -3,-1 -4,1 -2,-2 -4,-1 -6,0 -2,0 0,1 -2,-1 0,-1 3,-3 4,0 5,-2 m -224,-47 0,-1 1,1 -1,1 1,-1 -1,1 0,3 0,1 0,-3 0,-2 m 2,-8 1,4 -3,2 0,-1 1,-2 -1,0 1,-1 -1,-1 1,-2 1,0 0,1 m -10,1 -1,4 -3,3 -1,0 0,-2 2,-1 0,-1 2,0 0,-2 0,-1 1,-1 0,1 m 8,1 -1,5 0,5 -1,-1 1,-7 -1,-3 0,1 0,3 1,0 -1,7 0,-5 -1,-2 0,2 -1,3 0,3 -2,0 -1,2 2,-7 -1,0 -1,1 -2,6 -1,-1 -1,0 2,-6 2,-3 0,-2 -1,-4 1,2 0,2 0,2 0,3 1,-2 1,-5 -1,-4 1,2 1,0 1,1 2,0 0,2 1,0 m -9,-7 1,2 -1,0 0,1 0,-1 -1,-1 0,-1 1,0 m 90,-81 0,1 1,0 0,2 -1,0 0,1 -1,-3 -1,-1 1,0 1,0 m -106,-37 1,0 1,0 -3,1 0,-1 1,0",
                        "NS": "m 911,769 2,-1 2,1 1,0 0,1 1,-1 1,1 -1,-1 2,-2 2,0 -2,1 2,0 -1,1 3,-2 -1,2 1,0 1,-2 0,1 2,-1 -1,-1 6,-2 -1,0 2,0 -2,2 1,1 0,-1 0,1 1,-1 0,-1 2,-1 0,1 1,0 1,-2 -1,1 2,-2 2,-6 3,3 2,0 1,-1 0,1 1,0 0,-2 1,-1 3,2 2,0 0,1 -2,3 1,1 2,-2 4,-2 1,1 -1,0 0,1 0,1 -1,1 0,-2 -1,1 1,1 -1,-1 -1,1 -1,1 1,1 -1,1 -2,1 -2,0 2,1 -1,0 1,1 -3,1 1,0 -1,0 1,1 -1,0 -1,2 1,0 -3,2 -1,2 -1,0 0,2 -1,-1 0,1 0,1 -1,0 0,1 -2,2 1,1 -1,0 -1,0 -1,1 0,-1 -1,1 1,0 0,1 -2,-1 1,2 -1,-1 0,1 -1,0 0,1 0,1 -1,0 -1,0 1,1 0,1 -5,-1 1,1 2,0 1,2 0,1 0,1 -1,-1 -1,1 -1,0 0,-1 -1,2 -1,0 -1,-2 0,-2 -2,2 0,1 2,1 -1,0 0,1 -1,-1 -1,0 0,1 -2,0 1,2 0,1 1,0 1,0 -1,1 2,-1 -2,1 0,1 1,-1 0,1 1,0 -1,1 -2,-1 1,1 1,0 -1,1 0,2 -2,1 2,0 0,1 -2,2 1,0 -2,3 2,1 -1,0 0,1 -1,-1 1,1 0,1 -2,-1 1,1 0,1 -1,-1 1,1 0,1 -1,1 0,-1 0,1 0,1 -1,-1 0,-1 0,1 -2,-1 0,1 1,1 0,1 -1,-1 -1,-1 0,1 1,1 0,1 1,1 -2,0 2,2 -1,-1 -1,1 1,1 -2,-1 -1,2 -1,0 -1,-1 -1,-2 0,1 -2,-3 0,1 -2,-1 1,1 -1,-1 -1,0 2,2 0,1 -1,-1 0,1 -1,-1 0,1 -1,-1 0,-1 0,1 -1,-1 -1,-3 -2,-1 0,-3 -1,-2 2,-7 -2,1 -1,5 -1,0 1,-2 2,-6 2,1 2,-5 -2,4 -1,0 0,-1 8,-14 4,-5 0,-1 -1,0 2,0 0,2 1,0 0,1 1,1 0,-1 1,-1 2,1 0,2 1,-1 -1,-1 -2,-1 0,-1 3,-4 1,-1 3,-2 1,0 0,-1 1,-1 -6,2 -1,2 -1,-1 -2,1 -3,3 -3,0 -2,2 1,1 -1,1 0,-1 -1,0 -1,1 -1,-2 4,-7 -1,-2 1,-1 0,-1 1,2 0,-1 0,-2 0,-3 1,-1 m 41,-28 1,-3 0,-1 1,1 -1,5 -2,2 -1,0 0,2 -1,2 0,1 1,-2 1,-2 1,-2 0,2 1,1 -3,3 0,1 1,-2 1,0 -1,1 1,0 -1,1 0,1 0,2 1,-1 1,-2 2,0 0,2 1,-2 -1,-1 0,-1 2,-6 -2,4 -2,0 3,-9 -1,1 -1,3 -1,2 1,-7 1,-1 2,1 0,2 1,0 -1,-1 1,-2 1,0 0,1 1,-1 1,1 2,-1 -1,1 1,-1 -1,3 2,0 1,1 -2,1 1,1 -2,1 0,1 1,0 -1,1 0,2 -4,6 -2,0 -1,0 -3,4 0,-1 -1,0 0,2 -4,-2 -5,-5 1,-1 -1,-2 1,-3 1,-5 -1,-3 -1,0 0,-1 1,1 0,-4 1,-4 -1,-2 1,-1 1,0 1,-1 1,0 -1,3 1,-1 0,1 2,-1 1,3 0,1 1,0 -1,0 1,1 1,8 0,1 m -66,68 1,1 0,2 -1,2 0,-3 0,-2 m 69,-54 0,2 0,-1 -1,1 -1,-1 1,-1 1,-1 1,0 -1,1",
                        "SK": "m 403,590 0,1 -3,37 0,1 0,4 -1,7 -1,5 0,5 -1,13 -1,3 0,8 0,2 0,1 -1,4 1,1 -1,7 1,0 -1,7 1,0 -1,5 0,2 0,7 0,1 -1,13 1,0 -1,15 0,7 -1,7 1,0 -1,21 -1,7 1,0 -1,8 1,0 -1,7 0,7 -1,7 1,1 0,3 -2,0 -7,-1 -13,-1 -15,-2 -13,-1 -15,-2 -14,-3 -14,-2 -17,-4 -6,-1 21,-100 2,-9 1,-4 1,-3 6,-30 0,-1 12,-52 1,-7 3,-14 29,6 8,1 3,1 1,0 4,0 8,2 4,0 23,3",
                        "AB": "m 222,784 1,-1 -2,-2 1,-1 -1,-1 -2,-1 0,-1 0,-2 -1,-1 0,-1 1,-2 -2,0 0,-2 1,0 1,-2 0,-1 0,-3 0,-1 1,0 0,-2 0,-1 0,-3 0,-1 1,-1 -1,-1 -1,-4 -1,-1 -2,1 -1,-2 1,-1 0,-1 -1,0 0,-1 -1,-1 -1,-2 1,-1 0,-1 -2,-2 0,-1 -2,-1 0,-1 0,-1 0,-1 -2,-2 0,-2 0,-1 -1,-1 1,-1 -1,-1 0,-1 -1,-1 0,-1 0,-2 -1,-1 -1,1 0,1 -2,-1 0,-4 -2,-4 1,-2 -1,0 -3,-1 -1,-1 -1,-1 0,-1 2,-1 0,-1 -2,-1 -1,-2 -1,1 -1,0 -1,0 0,-1 1,-1 -1,-1 1,-1 -1,-1 1,-1 0,-1 0,-2 0,-1 0,-1 -2,0 0,-2 0,-1 -1,-1 1,-2 -2,-3 0,1 -1,1 -2,-2 -1,-2 0,-2 -2,-1 -1,-1 0,-1 -1,-2 0,-1 0,-1 2,1 0,-2 -1,-1 1,-1 -1,-1 1,-4 44,-113 97,28 -3,14 -1,7 -12,52 0,1 -6,30 -1,3 -1,4 -2,9 -21,100 -10,-2 -7,-2 -12,-3 -25,-6",
                        "NL": "m 955,625 -1,-1 0,5 -1,-1 -1,-2 -1,0 3,4 0,-1 1,1 -1,2 1,1 -1,1 1,0 0,3 -2,0 2,0 0,2 0,3 0,1 0,2 1,2 0,-1 1,3 -1,2 2,0 1,3 0,-3 0,-6 2,1 -1,-1 1,0 -1,-1 -1,-2 1,-3 0,-1 2,1 0,4 1,-3 0,1 1,-1 -1,-1 1,0 3,0 0,-1 1,0 1,-2 1,1 0,1 -3,5 -1,1 1,1 -1,3 1,-2 1,-2 0,1 1,-1 0,2 1,0 0,2 -1,1 0,2 2,-4 2,0 -1,1 1,-1 1,1 1,-3 0,2 1,0 -1,-2 1,-1 1,0 -1,1 1,-1 0,3 0,-1 1,1 0,-2 1,0 -1,1 1,0 0,-2 -3,-1 1,-1 1,0 0,-1 1,1 0,3 2,1 -1,3 1,0 0,1 1,0 0,1 1,-2 -1,1 -1,-1 0,-1 1,-3 -1,-1 1,0 1,0 0,2 0,-1 0,-1 1,0 0,-1 0,-1 -1,0 0,-1 2,2 0,-1 0,-1 0,1 0,-5 1,-1 0,2 1,-1 0,2 1,0 1,2 0,-2 -1,-1 0,-1 2,0 0,-1 0,1 0,-1 1,0 -1,-1 1,-2 1,0 0,1 0,-1 8,0 0,2 0,1 1,0 0,1 -2,2 1,0 -1,2 1,0 -1,2 1,-1 0,2 -2,2 0,1 0,-1 2,-2 0,1 0,-2 1,0 1,0 -1,1 1,0 0,1 -2,1 1,-1 1,1 1,0 0,1 0,-2 -1,1 0,-2 0,1 2,-1 1,1 0,-2 0,1 -2,4 3,-2 -1,1 1,0 -1,2 -1,2 -1,0 0,2 1,0 0,-1 1,-1 0,-2 1,-1 0,2 1,0 -1,0 0,-2 1,-2 1,0 -1,2 1,-1 0,-1 1,0 0,2 0,-3 -1,-2 1,-1 1,0 0,1 1,0 0,-3 0,-2 2,1 1,4 0,2 -2,1 0,1 1,1 -1,1 -1,3 -4,1 1,2 1,1 3,-2 0,1 -2,2 2,-1 1,-2 1,3 -1,1 1,-1 1,1 0,2 -2,-1 -1,1 2,0 1,1 2,1 0,-2 1,2 1,0 0,1 0,-1 1,0 0,-1 -1,-1 1,0 -2,-4 0,-1 0,-1 -1,-2 1,-2 1,-1 0,-3 2,0 -1,1 1,2 0,2 1,1 0,3 1,2 0,1 1,-1 -1,2 1,-1 1,2 0,-1 1,0 -1,1 1,0 1,0 0,-4 -1,-5 1,0 1,1 1,0 1,1 0,1 1,0 0,2 1,0 0,3 0,1 2,3 -1,1 1,-1 0,1 1,0 -1,1 1,1 2,3 -1,1 1,3 -1,1 -2,-1 -1,1 -1,-1 0,4 -1,0 -1,-1 -1,-3 0,-1 -1,3 0,-2 0,-2 0,1 -1,-1 0,1 0,-1 -1,1 0,-3 0,-1 -1,0 -1,-1 1,2 0,1 -1,-1 1,2 -1,1 0,4 0,2 -2,1 -1,-2 -2,-6 1,-1 -1,0 -1,-1 1,0 0,-2 -1,1 0,-1 1,-1 -1,1 -1,-1 -1,-2 0,-1 -1,1 -3,-3 -1,1 -1,-1 0,1 -2,0 1,0 1,1 0,2 -1,1 1,0 0,1 1,3 1,0 0,1 -1,0 0,2 -1,0 0,-5 0,5 0,1 -1,0 0,1 -1,0 0,-1 0,1 -1,-1 0,5 1,1 0,1 -1,-1 0,1 2,1 -1,0 1,2 -1,-1 -1,0 1,1 1,2 -2,1 1,1 -1,1 -1,-1 0,1 -1,-1 0,1 0,1 -1,1 -3,1 -1,0 -1,-3 4,-5 1,-3 -1,-3 0,-1 2,-3 0,-2 0,-1 1,-2 -1,0 -1,3 -2,2 0,-5 0,5 -1,0 0,-1 0,1 -2,0 -1,1 0,-2 0,1 -1,0 0,1 1,1 0,1 1,-1 1,2 -1,1 0,2 -1,-1 -1,1 0,-2 0,1 -1,0 1,1 -1,0 -1,-1 1,2 -1,0 -1,1 1,-4 -3,5 -1,0 3,-5 1,-2 -2,2 -1,0 0,-2 0,-1 -1,-1 0,-2 -1,0 1,3 -1,2 0,2 -1,0 0,-2 0,1 -2,-1 1,1 0,1 1,0 0,1 0,1 -1,1 -2,-2 1,1 1,1 -4,3 0,1 -1,-1 0,1 1,0 0,1 -1,1 -1,-1 -2,1 -1,1 -1,0 0,-1 0,1 -1,1 -1,0 -2,1 -1,0 0,2 -2,0 -2,1 0,-1 -1,1 -1,0 0,1 -1,-1 -2,3 -1,-2 0,3 -2,0 1,1 -1,1 -1,1 -4,4 -3,0 -1,-3 -1,1 -3,-2 1,-2 0,-2 2,-3 1,-9 1,-1 0,2 0,-1 1,-2 1,-1 -3,2 -1,-1 -1,1 -6,6 0,-2 2,-7 0,4 1,0 0,1 0,-1 0,-1 1,1 1,-1 -1,-4 0,-2 -1,-3 0,-4 1,2 2,-2 2,1 2,0 0,-1 -1,1 -2,-1 -1,-1 2,-2 0,-1 0,1 -1,0 0,-1 -1,2 -1,0 2,-1 0,-2 -1,2 -2,1 -1,-1 -1,-2 0,-2 0,-3 2,0 0,2 1,0 -1,-1 0,-1 2,0 1,0 -2,0 -1,-1 0,1 -2,-1 0,-1 -2,-11 -2,-4 0,-5 2,-2 -1,1 -1,0 0,-1 -2,1 0,-1 2,-1 0,-2 1,-3 -1,0 1,-1 -1,1 0,-1 -2,0 1,-1 0,1 1,-1 -1,-1 0,-1 1,-1 -1,-1 0,-1 -1,0 0,-2 4,-8 1,-3 1,-1 0,2 1,0 0,1 3,-2 -2,0 0,-2 0,1 1,0 1,-2 1,0 0,1 1,1 -1,0 1,0 1,2 -1,0 0,1 0,1 -2,1 -4,2 1,1 1,-1 0,1 0,1 1,0 1,0 1,-2 2,0 -1,1 1,0 1,1 -1,1 1,0 0,1 0,1 0,1 1,1 m -68,-55 0,1 1,1 1,0 0,-1 0,1 -2,2 -2,-1 2,-3 m -39,0 1,0 0,1 -1,1 0,-1 -1,0 0,-1 1,0 m -5,-2 2,0 0,1 1,0 -1,1 -2,0 -1,0 1,-1 -1,0 1,-1 m -7,-2 1,0 0,1 1,0 -1,0 -2,1 -2,-1 3,-1 m -2,-2 3,-1 0,1 2,-1 -1,0 1,0 -3,2 -2,0 2,-1 -2,1 0,-1 m -2,-8 0,1 1,0 0,1 -1,0 0,1 0,1 1,-1 2,1 -1,1 1,0 -1,1 -3,-1 -1,-2 1,-2 -1,-1 1,0 1,0 m -9,-9 0,1 1,0 -1,1 1,0 1,0 -3,1 0,-2 0,-1 1,0 m -2,3 -1,-1 0,-1 0,-1 0,1 0,-1 2,0 0,1 -1,2 m 0,-7 0,1 -1,0 -1,-1 0,1 0,-2 1,0 0,1 0,-1 1,1 m -3,-1 0,1 0,1 -2,0 0,-2 1,0 1,0 m -40,-24 1,0 0,1 1,0 0,1 -1,1 0,-1 -1,1 0,-1 0,-1 0,-1 m -8,-6 2,0 -3,0 -1,2 2,-1 1,0 0,1 1,-1 0,1 0,1 0,1 -1,1 -1,2 1,-2 2,-1 0,-2 0,1 0,1 1,0 -2,2 2,-1 0,1 0,1 1,-1 0,-1 0,-1 -1,0 0,-1 1,0 1,1 -1,1 1,0 0,1 1,-1 1,3 -1,0 2,1 2,-1 0,1 -1,1 1,2 0,-1 0,-1 1,-1 0,3 1,-2 1,0 1,2 -2,1 0,1 2,-1 1,0 0,3 1,-1 0,-2 1,-1 1,2 -1,1 2,-2 0,1 0,1 1,-1 0,2 -1,1 -1,2 -2,0 -1,2 1,-2 1,1 0,1 0,-1 3,-2 1,-2 2,-1 0,1 -1,1 1,0 0,1 1,-1 0,1 -1,2 2,-1 0,1 0,-2 1,0 0,1 1,0 1,0 0,1 0,1 1,-1 1,1 -1,1 1,1 -2,3 -2,1 -1,1 1,-1 1,0 -1,4 2,-4 2,-1 0,2 0,-2 1,0 1,-2 1,0 -1,0 2,-1 0,2 1,0 0,1 -1,2 0,1 0,-1 1,-1 1,0 1,1 -1,1 -3,2 0,1 -1,2 1,-1 0,1 -1,2 2,-2 -1,-1 2,-3 0,1 1,-1 1,0 0,-2 1,1 0,-1 0,2 1,0 0,-1 1,-1 -1,1 1,0 0,1 0,-1 1,1 0,1 -2,0 0,2 1,-2 1,0 1,2 -1,-2 1,1 0,-1 1,0 0,-1 2,1 0,1 0,1 2,-1 1,1 0,1 1,0 -1,2 0,1 -2,1 0,2 -2,0 3,1 1,-2 3,-1 2,1 -2,0 1,0 1,1 -1,2 1,0 1,0 1,-1 0,1 0,-2 1,0 0,2 2,-2 2,1 0,1 0,1 1,0 -1,0 -1,1 0,2 0,2 -2,1 1,2 0,-1 2,-1 0,1 -1,0 2,0 0,1 -2,1 0,-1 -1,1 -6,1 1,-2 2,-2 -4,3 0,1 1,0 7,-1 -3,2 3,-1 2,-1 1,0 1,1 -1,1 -3,1 0,1 1,-1 -1,2 1,-1 1,0 1,0 1,1 2,-1 -2,1 2,0 -1,1 -1,0 1,-1 -1,1 -2,1 2,1 2,-2 2,0 1,-2 1,1 0,1 1,0 -1,2 2,-1 -2,2 1,-1 1,1 -1,0 1,0 2,-2 1,1 1,0 -2,1 2,0 3,-2 1,1 0,1 -2,1 1,0 1,-1 1,-2 0,1 1,1 0,1 1,0 0,2 0,-2 -1,-3 2,-1 0,1 1,1 -1,1 0,1 0,-1 1,0 1,3 0,-2 0,-1 1,1 -1,3 1,0 0,1 0,-3 1,1 0,-2 1,0 0,1 -1,2 1,2 -1,0 0,2 0,1 0,-1 0,-1 1,0 0,-1 0,2 0,2 -1,2 1,1 1,-3 -1,-1 1,-1 1,-3 1,1 1,-2 0,4 -1,0 0,1 0,1 1,-3 0,-2 1,-1 0,-1 0,-1 1,0 1,0 0,1 0,1 -1,1 -1,1 1,0 1,-1 1,-1 1,0 0,-1 0,-1 1,1 1,0 0,2 -1,3 -1,2 1,0 0,1 0,2 1,-2 -1,-1 0,-1 1,-2 1,-3 0,-3 0,-1 1,1 -1,-2 1,1 1,1 -1,1 0,3 1,-2 -1,-1 1,0 0,-1 1,1 -1,-2 1,0 1,0 0,3 0,-1 2,1 0,1 1,-1 0,1 1,0 1,0 0,-1 1,1 2,-1 -1,1 2,-1 0,-1 2,-1 1,1 2,-1 1,-1 3,0 1,-1 1,0 1,0 1,0 -1,1 1,0 0,1 -3,2 3,-1 -1,0 1,1 -1,2 -5,3 -1,2 2,-2 -1,2 -2,2 -1,3 -2,2 -4,4 -2,1 1,1 2,-2 5,-5 2,-4 0,4 -4,3 1,0 0,2 -1,0 -2,5 -4,2 -1,0 1,0 0,1 -3,2 2,0 0,1 0,2 1,0 0,1 -4,-1 -6,1 -1,-1 1,1 -1,2 1,0 1,-1 3,-1 4,0 3,0 1,1 -1,0 -2,3 0,1 2,-1 -1,2 1,-1 1,-1 1,0 -1,0 1,-2 0,-2 0,-1 1,0 -2,0 3,-2 1,-4 1,-1 1,-1 1,0 0,-1 0,-1 -1,0 1,-3 0,-1 5,-8 4,-1 -1,-1 -4,2 -1,2 -1,0 0,-2 1,-2 6,-2 0,-1 1,1 1,-2 1,1 1,0 2,1 2,1 -1,0 1,1 2,0 -1,0 1,1 0,1 -2,5 0,-1 1,0 1,0 0,1 1,0 0,2 1,-1 0,-1 0,-2 0,-1 0,-1 0,-1 2,-3 2,0 -1,-1 1,-1 2,0 -3,3 3,-1 2,-1 1,0 -1,1 3,-2 1,0 0,1 -1,0 -2,1 4,0 1,0 0,2 0,-2 1,0 0,1 1,-1 0,2 1,-1 0,1 1,0 0,1 -1,0 1,0 1,1 0,1 -2,0 1,0 1,1 -3,2 1,0 0,1 1,-1 2,-1 0,1 1,0 1,1 -1,2 -2,1 1,0 2,1 2,-2 1,1 0,1 -2,0 -2,1 2,0 0,1 -1,0 -1,2 -3,1 3,-1 1,-1 2,0 3,-2 0,1 1,-1 1,2 -2,0 1,1 -5,1 5,0 1,-1 0,2 1,-1 1,0 -1,1 1,0 0,1 1,0 -1,1 1,1 -1,1 -1,1 1,0 1,1 -2,3 0,1 -2,3 0,1 -1,2 0,1 0,2 0,1 -1,1 1,1 -2,1 -7,-10 -2,2 -70,41 -1,0 2,-1 -1,-1 -4,-1 -2,-2 1,0 -1,-1 -1,0 -1,0 0,-3 1,-1 5,-3 -1,-1 -2,0 -2,-4 -1,-1 -1,-1 -1,0 -2,2 1,2 0,1 1,1 -2,0 -1,2 1,1 1,0 0,2 1,1 0,1 3,3 0,2 2,2 -2,1 1,2 0,1 0,1 1,1 0,1 2,1 -1,3 -1,2 -1,-1 -1,0 -1,-1 1,-2 -4,-1 -1,-1 -5,-2 -4,2 0,2 0,1 -2,0 -1,1 1,1 -2,0 -2,0 -1,-2 -1,0 -2,-1 0,1 1,1 0,2 -2,-1 -1,-2 -1,-1 1,-1 -2,-2 1,-1 -1,0 -1,-2 1,-1 -2,-2 1,-1 -2,-2 -2,0 1,2 -2,1 2,4 -1,2 -2,-2 1,2 -2,0 -1,0 0,-2 -2,-2 1,-2 -1,-2 -1,-1 0,-3 -2,-1 0,1 -1,-1 -1,-1 0,-1 -5,1 -1,-4 -2,0 -1,-2 -2,0 -1,-1 1,-3 1,-2 -1,0 0,-1 -2,-1 0,-1 -1,-1 0,-2 1,0 2,0 0,-2 2,-1 -2,0 0,-1 -2,-1 0,-1 -1,0 -3,-3 0,-1 2,0 0,1 4,0 2,2 3,0 -1,-1 1,-1 -1,-1 -1,-1 0,-1 -2,-2 0,-1 0,-1 -3,-1 1,-1 1,0 1,1 1,0 6,3 2,0 1,-1 1,0 1,2 3,1 2,-2 4,-1 3,-2 3,-1 1,-2 1,0 5,0 3,-1 -1,-8 -2,-1 -1,-2 2,-4 1,-2 -2,0 0,-2 -2,0 -2,0 -2,-2 -3,-1 -1,-2 2,-3 -4,2 -1,0 0,-1 -2,1 0,-1 0,-2 -1,0 -1,1 -1,-1 0,-1 -1,-1 2,-1 0,-1 -1,0 0,-1 -4,-2 0,-2 1,0 -1,-2 1,-1 -1,0 0,-2 -1,-1 1,-1 -1,-1 0,-2 -1,0 0,1 -1,-2 1,-1 -1,-2 -1,-2 0,-1 0,-1 0,-2 -1,1 0,1 -1,-1 0,1 -1,0 -1,-1 -1,0 0,1 -4,-3 0,-1 -3,0 0,-2 1,-1 0,-3 2,-4 -2,-1 -1,2 -1,-1 0,-1 -1,-1 4,-3 1,-1 -2,-1 -2,0 0,1 -1,1 -1,2 -1,0 -1,-1 -1,-1 -2,2 -2,-1 -1,1 -1,0 0,-1 0,-1 0,-1 1,0 3,0 1,-1 -1,-1 -2,0 0,-1 -1,0 0,-1 -1,-1 0,-1 -1,0 1,-1 -1,-1 1,-1 -1,-1 -2,3 -2,-1 -1,-3 -1,-2 0,-1 0,-1 0,-1 -2,0 -1,-1 1,0 0,-1 0,-1 1,0 -1,-1 -1,0 -1,0 -1,-1 3,-2 m -2,-1 -1,0 -2,0 0,-1 2,-1 2,1 0,1 0,-1 1,0 0,1 1,1 -3,-1 m 245,154 1,5 -1,1 0,-2 0,-2 -1,0 0,-2 1,1 -1,-2 1,1 m -21,14 0,1 1,0 -3,1 2,-2 m 20,-28 1,-1 0,1 0,1 1,0 -1,1 -3,2 -2,-1 0,-1 1,1 1,-1 1,0 1,-2 m -28,-16 0,1 -1,2 0,1 -1,1 0,-1 -1,0 -1,-1 1,-1 1,1 1,-2 0,-2 1,1 m 3,-5 2,0 -2,4 -1,-2 -1,0 1,-1 1,0 -1,-1 1,0 m -27,-7 0,2 -1,0 -1,-1 1,0 1,-2 0,1 m -36,-42 2,-1 -1,0 1,1 -1,0 1,1 -3,0 1,-1",
                        "BC": "m 183,772 -5,-2 -10,-3 -8,-3 -25,-10 -24,-10 -1,-1 0,-1 -2,0 -1,1 -1,-2 3,-1 -2,0 -1,-1 1,-1 1,1 -1,-3 1,1 0,-1 3,2 2,-3 -1,2 -1,0 -4,-2 0,-1 1,0 1,-2 3,-3 -2,0 -1,1 -3,1 -1,1 -1,1 -2,-3 -1,-1 -1,-4 1,-1 1,0 1,3 -1,2 1,-2 2,1 2,-1 -4,0 0,-4 1,-1 0,-3 4,-1 -1,-1 -1,-2 1,3 -3,1 -1,1 1,2 -1,0 -1,0 1,-2 -1,1 -1,1 -2,-1 -1,1 -2,-2 0,-1 -1,-4 -1,-2 1,2 1,-1 0,-1 2,-1 1,-1 -1,-2 1,-1 3,0 2,-1 -3,1 -2,-1 -2,1 -2,-1 1,-1 0,-1 -1,2 -1,0 1,-3 1,0 3,-1 0,-2 1,0 0,-2 2,-1 0,-1 -2,1 -1,0 0,2 -1,1 0,1 -3,1 -1,1 -1,0 0,-1 0,-1 -1,1 0,-2 -1,1 -2,-1 1,-1 1,-2 2,-1 -2,1 -1,2 -1,0 0,1 -2,0 0,-2 1,0 -1,0 -2,0 -4,-3 1,0 0,1 1,-1 3,0 -3,-1 -1,-1 7,2 1,0 1,-1 1,-1 1,0 1,-1 0,-2 0,-2 -1,3 1,1 -3,0 0,1 0,1 -1,0 -1,1 -2,-1 -1,-1 -3,0 1,-1 3,0 -2,-1 1,-2 -5,-1 0,-1 1,0 3,1 1,0 -3,-2 1,-2 -1,0 -1,2 -2,0 -2,-2 -1,0 -2,-2 0,2 2,1 -2,0 -1,-3 -2,-1 0,-1 -1,-1 1,-2 1,2 1,0 1,1 5,2 1,2 0,-1 0,-1 2,-1 -3,1 -5,-2 -2,-1 0,-2 -2,0 -1,-3 1,-1 3,0 2,2 3,0 1,-1 -2,1 -1,0 -3,-2 1,0 -1,0 -3,-2 1,0 1,0 2,-1 2,-2 4,1 6,3 1,-1 1,-2 -1,2 -1,1 -4,-3 -2,-1 -1,-1 -1,0 0,-1 1,0 1,-2 -1,2 -1,-1 -1,1 -2,0 2,1 -1,0 -1,1 0,1 -2,1 -1,-1 2,0 -2,-1 1,-1 0,-2 2,-4 1,-1 2,1 1,-2 1,-1 2,0 -1,1 -1,0 -1,1 1,-1 2,1 0,-2 3,0 2,-1 1,1 1,2 -1,1 1,3 0,2 1,-1 0,-2 -1,-2 0,-2 1,-1 2,0 -3,0 -1,-1 -1,-1 1,-3 2,-1 2,0 1,-1 1,-2 -1,-2 0,4 -2,1 -3,0 -1,2 -3,0 -3,1 -1,-1 1,-1 -3,2 0,-2 1,-1 1,0 0,-2 1,0 -1,-1 -1,1 0,2 -1,0 -1,-2 1,-1 1,0 1,0 1,-1 -3,0 -1,2 -3,1 -1,-1 3,-2 1,-2 3,-2 1,-2 0,-1 2,0 -1,-1 -3,2 -2,-2 0,-1 2,-4 0,-1 1,0 -1,0 0,-2 -1,0 -1,-2 1,-1 0,-1 0,-1 2,-1 -2,0 1,-2 1,1 1,0 0,2 0,1 0,-1 1,1 0,2 0,1 1,-1 -1,0 2,0 1,-1 2,2 0,1 -1,1 1,0 1,1 -1,2 1,0 -1,-1 1,-1 -1,-2 0,-2 -2,-1 -1,0 -2,0 -1,-4 0,-1 2,-2 0,-1 1,0 1,0 2,1 -2,-2 2,-2 -1,0 -1,1 -4,2 1,-2 0,1 0,-1 0,-1 0,1 -1,1 -1,0 0,-1 0,1 -3,1 -2,2 0,1 0,1 -2,0 -1,-5 0,-2 0,-1 -1,-4 1,-1 -1,0 0,-1 1,-3 1,-1 1,1 0,3 1,2 0,-1 0,-2 0,-3 1,0 2,2 2,0 -2,0 -2,-2 -3,0 -1,-2 0,-1 1,0 0,-1 1,-1 -1,0 -1,0 -1,-1 1,-1 1,0 0,-1 1,-1 0,-1 1,7 1,1 -1,-1 0,-1 2,-2 1,0 -1,0 -1,1 -1,0 0,-5 0,-1 3,1 1,2 1,1 -1,-1 0,-2 -1,-1 1,-1 2,-1 -2,0 3,-1 0,1 2,0 2,1 -3,-3 1,-1 2,-1 1,-1 1,0 2,-2 2,0 1,0 -2,-1 -1,1 1,-5 -2,3 -1,2 1,0 -1,0 -1,1 -2,2 -3,2 -1,0 1,-2 4,-2 0,-2 1,-2 0,-2 1,-2 0,-2 3,-2 2,-4 -3,-2 0,-2 -1,-1 0,-3 -3,-2 -3,-7 -2,-1 2,-4 -2,-2 3,-3 -3,-3 2,-1 2,-21 0,-4 1,-4 -1,-6 -2,-5 1,-6 -1,-1 1,-2 -2,-2 1,-2 -1,-2 1,0 1,-2 -1,-6 -9,-2 0,2 -3,0 -2,3 -3,1 -2,-1 -8,0 3,-6 -2,-15 1,-3 -1,-3 33,24 2,1 5,3 3,2 78,43 11,6 36,15 -44,113 -1,4 1,1 -1,1 1,1 0,2 -2,-1 0,1 0,1 1,2 0,1 1,1 2,1 0,2 1,2 2,2 1,-1 0,-1 2,3 -1,2 1,1 0,1 0,2 2,0 0,1 0,1 0,2 0,1 -1,1 1,1 -1,1 1,1 -1,1 0,1 1,0 1,0 1,-1 1,2 2,1 0,1 -2,1 0,1 1,1 1,1 3,1 1,0 -1,2 2,4 0,4 2,1 0,-1 1,-1 1,1 0,2 0,1 1,1 0,1 1,1 -1,1 1,1 0,1 0,2 2,2 0,1 0,1 0,1 2,1 0,1 2,2 0,1 -1,1 1,2 1,1 0,1 1,0 0,1 -1,1 1,2 2,-1 1,1 1,4 1,1 -1,1 0,1 0,3 0,1 0,2 -1,0 0,1 0,3 0,1 -1,2 -1,0 0,2 2,0 -1,2 0,1 1,1 0,2 0,1 2,1 1,1 -1,1 2,2 -1,1 -9,-3 -17,-5 -13,-4 m -83,-31 1,3 -1,-1 1,3 -1,-1 0,1 -1,-1 0,-1 0,-1 1,0 -1,-1 1,-1 m 0,-4 1,4 2,3 -1,0 -1,-3 -1,-2 0,-2 m -27,-19 0,1 -1,2 -2,-2 1,-1 2,0 m 25,9 -3,-4 0,-1 -1,-2 0,-1 3,3 1,5 m -28,-22 0,1 0,2 -1,2 0,1 -2,-1 -1,-4 2,0 -2,-1 1,0 3,0 m 24,4 0,3 -2,1 0,-1 0,-1 2,-1 0,-1 m -4,-3 1,0 1,2 -1,1 -1,1 0,1 0,1 0,-2 -1,-1 1,-3 m 6,2 0,1 0,1 0,1 -1,1 0,-1 0,-1 -1,0 0,-1 2,-1 m -4,-4 1,1 0,4 -1,0 -1,1 1,-2 0,-2 -2,-1 0,-1 1,1 0,-1 1,0 m -1,-2 1,1 -2,1 -1,0 0,-1 2,-1 m -10,-6 -4,-2 5,1 0,1 -1,0 m 3,-5 0,2 -1,0 0,1 -4,-3 4,-1 1,1 m -7,-3 0,-1 1,1 0,-1 2,1 0,1 -3,-1 m -12,15 0,-2 1,-1 0,-1 2,0 0,-1 -1,1 -1,-2 -1,2 -1,0 -1,-2 1,-1 -1,0 2,-1 -3,1 0,-1 0,-1 -3,1 -1,-1 3,-2 0,-1 1,0 -2,-1 0,-1 1,-2 5,2 0,3 0,-3 2,-1 1,0 -2,0 0,-1 -3,-2 0,1 2,1 0,1 -5,-1 -1,-1 0,1 -1,-3 0,-1 -1,-1 1,-1 -1,0 1,-2 5,0 2,1 3,4 0,2 1,-1 0,1 1,3 4,3 0,1 1,0 1,2 3,2 3,2 1,2 2,1 2,1 1,2 0,3 -1,1 1,0 0,5 1,3 1,2 -1,1 -1,-1 1,5 0,1 2,2 4,4 0,1 -1,0 3,1 0,3 1,0 0,2 -1,1 -1,-1 2,5 -1,0 1,1 0,1 -1,-1 0,2 0,2 -1,1 1,0 1,-2 0,-1 1,1 -1,3 1,2 -1,1 -1,-2 -3,2 -2,-2 -2,-1 -5,-6 1,-1 -2,0 -2,-2 -1,-2 -3,-3 0,-2 3,0 0,-1 3,-1 2,-1 0,-3 -1,2 -2,2 -2,0 -1,-1 -1,0 -1,-1 1,0 -1,-1 -1,1 -2,0 -2,-4 -1,-1 2,0 2,-1 -2,1 1,-2 -1,-1 2,-2 -2,2 -1,-1 -1,1 0,-1 1,-1 2,-2 -2,2 -1,-1 1,-2 -2,0 0,-1 -2,1 0,-1 0,-1 -1,0 0,1 -1,0 -1,-1 1,-1 1,-2 2,-1 4,2 1,0 -3,-1 -2,-1 0,-1 2,-1 -1,0 -1,1 -1,-1 1,-4 -1,0 0,-2 -1,1 -1,0 1,-1 -2,1 -2,-1 m 8,-39 -1,5 -1,1 -1,-1 -1,-1 1,-3 1,-1 2,0 m 3,-6 -1,2 -2,2 0,1 -2,0 1,-1 0,-1 -1,-1 1,-1 0,-1 2,-1 2,1 m 1,-2 1,1 -2,1 -1,-1 0,-1 2,0 m -2,-1 -1,2 -1,0 0,-2 1,-1 1,1 m 4,-1 -1,2 -2,-1 1,0 0,-1 -1,-1 2,0 1,1 m -1,-3 0,2 -2,0 2,-2 m 3,8 -1,0 -2,2 -2,0 0,-1 1,-2 3,-2 4,0 2,-1 2,0 0,1 -1,1 0,1 -6,1 m -8,-12 -1,2 -2,1 0,-1 1,-3 2,0 0,1 m 3,-3 -1,4 -1,0 -1,-2 -1,0 -1,-1 1,-1 1,1 0,-1 2,1 1,-1 m -31,-19 -1,1 -1,-1 -1,0 1,1 -1,0 0,-3 2,1 -2,0 1,1 2,0 m 34,17 -2,6 -2,2 -1,0 0,-1 1,-1 1,-1 0,-2 2,-1 0,-1 1,-1 m -9,-5 1,6 -2,2 0,-2 -1,-1 1,-1 -1,-1 0,-1 1,1 -1,-2 1,-1 1,0 m 11,6 -2,3 -1,0 1,-3 1,0 1,0 m -3,-3 0,2 -2,2 0,-1 2,-3 m -7,-11 0,3 0,2 -1,-2 0,-2 -1,-1 1,0 1,0 m -27,7 0,1 -1,0 0,-1 -1,1 -1,0 1,-1 0,-2 -1,2 0,1 0,-3 -1,1 0,-3 0,-1 0,-1 1,0 -1,-1 0,-1 -1,-2 0,-2 0,-1 -1,-2 1,-1 0,1 1,-1 0,2 0,-1 0,-2 -1,1 -1,0 0,-2 0,-1 0,-2 1,1 0,1 0,-1 1,1 0,-1 1,0 -2,-2 -2,-1 0,-2 1,0 0,-1 1,2 4,1 1,0 3,0 0,2 -1,0 1,2 0,1 -2,-1 -2,-1 0,1 1,0 2,2 -1,1 -1,1 -1,-1 -1,-2 0,1 -1,0 1,0 0,1 -1,0 1,1 2,1 -3,0 0,1 1,1 0,1 -1,0 1,1 -1,1 1,2 0,1 -1,1 1,0 0,1 1,-1 1,0 -1,1 0,1 -1,-1 0,1 0,2 1,-1 1,0 0,1 -1,0 1,1 m 31,8 -2,0 1,-1 0,-2 3,-2 0,-1 0,1 -1,-1 0,1 -2,2 -1,1 -1,0 -1,-3 0,-2 1,1 0,-1 2,-1 1,-1 1,0 -1,-3 0,1 1,2 -1,0 -3,1 -1,-1 1,-2 1,0 1,0 2,-3 0,-1 2,1 1,4 1,1 -1,2 0,2 -4,5 m 0,-15 -1,3 -2,0 0,-1 0,-1 2,-2 0,-1 1,1 0,1 m 4,-3 0,3 -1,1 -2,-1 0,-2 2,0 1,-2 0,1 m -14,-10 3,6 0,5 -2,0 0,-2 -1,-1 1,-1 -1,-1 0,-1 -1,-1 0,-2 1,0 -1,-1 -1,-1 1,-1 1,1 m 18,7 -1,1 -1,1 -1,0 -2,0 0,1 -2,1 3,-4 2,0 2,0 m -15,-10 1,1 -1,1 1,1 -1,2 -2,-2 0,-2 2,0 0,-1 m 3,15 -1,0 0,-2 0,-1 -1,-2 1,-1 -1,-1 1,-1 0,-1 -1,0 0,-1 1,0 1,0 -1,0 -1,-1 0,-1 -1,-1 1,-2 0,-1 2,0 0,2 2,9 -1,0 1,1 1,1 -2,2 -1,0 0,1 m 0,-20 0,2 -3,2 -1,0 1,-1 -1,0 0,-1 1,0 1,-1 1,1 0,-1 -1,0 -2,1 -1,-2 -1,1 0,1 -1,-1 1,-1 1,-1 4,0 1,1 m -32,-8 0,-2 1,1 -1,-1 1,-1 -1,0 0,-1 1,-1 0,-1 0,-1 1,0 0,-1 1,0 1,-2 1,0 0,-1 1,3 2,0 1,1 -1,2 -1,0 -1,1 4,-1 2,0 0,1 0,1 0,1 -1,2 -2,1 -6,-1 1,1 -1,1 1,-1 1,1 1,1 -1,0 -1,1 1,-1 2,1 -1,-1 1,0 1,1 1,-1 0,-1 3,-2 0,-2 0,-1 2,2 4,0 1,-1 -3,3 -3,2 -1,2 -2,1 -2,5 -2,1 -3,-1 -1,0 1,1 1,0 -1,0 -1,-1 -1,1 -1,-1 0,-1 -1,0 1,0 -1,-1 1,0 -2,-2 2,0 1,0 -2,-1 1,-1 1,2 1,2 0,-1 0,-2 -1,-1 0,-1 -1,0 1,-1 -2,0 1,-1 -1,-1 1,0 -1,0 0,-1 1,0 m 33,-4 -1,1 -3,0 2,-2 2,1 m 5,-2 0,1 -1,1 -1,0 0,-2 2,0 m 5,-2 -1,2 -2,1 -2,0 0,-1 4,-2 1,0 m -39,45 1,2 -1,0 -1,0 0,1 0,1 -1,0 0,-1 1,-1 -1,0 1,-2 1,0",
                        "NB": "m 835,776 5,-6 1,-3 -1,-2 -2,-5 3,-2 0,-1 6,-3 1,1 4,1 0,-2 1,0 2,-2 1,0 0,-1 1,-1 1,-1 4,-4 1,1 8,-1 4,3 0,1 0,-1 2,-1 1,-4 3,-2 0,2 3,-2 0,1 1,0 0,-1 1,0 -1,2 1,1 -1,2 0,1 1,-1 0,2 0,-1 1,1 0,2 0,-1 -1,0 1,2 -3,6 2,-1 0,1 2,-2 1,1 0,-1 2,-1 0,1 1,1 -1,1 1,2 1,1 1,0 0,1 0,1 0,-1 1,0 1,0 1,2 1,1 2,1 0,2 -1,0 2,-1 1,0 -1,1 1,1 4,-2 2,0 0,1 0,-1 1,0 1,-1 3,0 -1,2 -2,1 0,1 1,0 -1,1 0,3 -2,0 0,4 -1,0 0,-1 -1,-2 0,1 -1,0 -2,-2 -1,-1 -1,0 1,1 3,2 0,3 1,0 -1,0 0,3 -2,1 0,2 -4,4 -3,6 -1,0 -1,2 -1,0 -1,-1 -2,1 1,-2 -1,-2 0,2 -1,1 1,1 2,0 -2,3 0,-1 -1,1 1,0 -1,1 -1,2 -1,-1 -2,2 -1,0 0,1 0,1 -1,-1 -1,1 0,-1 0,-1 -2,0 0,1 -1,0 1,1 -2,0 0,-1 -1,-1 1,3 0,-1 -2,0 0,2 -2,-1 -2,-1 0,-3 -2,-1 1,-1 -1,-2 -2,1 -2,-1 -1,1 -1,-2 0,-1 -1,-1 0,-1 -10,-21 -8,-3 -2,0 0,2 -2,1 -4,4 -2,-1 m 52,-40 0,2 0,2 0,1 -1,0 0,-1 -1,0 0,-1 1,-1 1,0 0,-2",
                        "PE": "m 908,761 -2,1 0,-1 -1,-2 0,-1 -1,-1 0,2 -2,-1 -2,2 0,-1 0,-6 1,-4 2,3 0,3 2,0 2,1 0,1 1,-1 0,1 0,2 1,-1 0,1 2,0 0,-1 -1,0 0,-1 0,-1 3,-1 0,1 1,0 0,-1 -1,0 2,0 2,1 0,-1 8,-4 -1,0 2,-2 4,-2 3,-1 -2,3 -2,1 1,0 -1,1 1,0 -1,1 1,0 -2,1 1,0 0,1 -2,1 2,0 -2,0 1,0 -1,1 1,-1 1,1 1,0 -1,0 0,-1 1,1 -1,2 2,0 -3,3 -2,0 -1,-1 -2,1 1,-2 1,-1 -1,0 -1,1 0,-1 -1,0 1,0 -2,1 0,-3 -1,2 -1,0 1,1 -1,1 1,0 0,-1 1,0 -1,2 -4,0 -1,1 -2,0 -2,-1 -1,-1 2,0 -1,0 -2,0 -1,0 0,1",
                        "YT": "m 190,534 -11,-6 -78,-43 -3,-2 -5,-3 -2,-1 -33,-24 -1,-3 4,-4 -5,-3 -4,0 -2,-4 -2,0 -3,-4 21,-25 51,-62 33,-41 16,-20 3,4 3,2 2,4 1,1 -1,2 1,2 -1,3 1,0 1,0 0,-1 0,2 0,2 3,6 -1,0 1,1 1,1 1,3 2,1 -14,19 -1,2 1,1 -1,2 -1,2 0,1 -1,1 -1,1 -1,0 0,1 -1,1 -1,2 16,10 -2,-1 0,1 0,1 0,3 -1,1 0,1 -1,1 1,2 -1,0 -1,2 -1,-1 -1,1 -1,1 2,2 -2,2 -1,0 -1,1 1,0 -1,2 1,1 3,0 1,1 0,1 -1,0 1,1 3,-1 0,1 0,1 1,0 0,1 -2,0 -1,1 1,3 -1,3 -1,0 -2,2 -2,0 -1,1 -2,0 0,1 -1,0 1,1 1,0 0,1 -1,0 0,1 1,1 0,1 -2,0 -1,2 -1,0 -1,1 2,1 1,2 0,1 0,3 1,1 0,1 -1,1 -1,1 1,1 2,0 1,-1 1,2 1,2 -1,1 0,1 0,1 -1,1 0,1 0,1 0,1 1,2 0,2 2,2 0,1 -2,0 -1,0 1,1 1,0 0,1 -1,0 1,1 0,1 0,2 -1,0 -1,2 -1,0 -1,0 -1,1 1,0 0,2 1,0 -1,2 2,1 0,1 -2,1 -1,1 -1,0 0,3 0,1 0,1 0,1 -1,1 1,0 1,3 -1,0 0,2 -1,0 -1,1 1,2 -2,0 1,2 2,0 1,2 1,0 1,0 1,3 -1,0 1,2 -1,2 1,0 0,3 0,1 2,5 2,1 1,1 -1,2 1,1 -2,3 0,1 -1,1 1,1 -1,1 0,1 -1,1 0,1 0,1 0,1 1,-1 0,2 2,-1 1,1 1,1 1,-1 0,-1 1,0 0,1 0,1 1,0 1,-1 1,0 1,2 2,3 3,-1 1,1 1,0 1,-1 1,-1 2,2 -1,3 -2,1 0,1 1,2 -1,2 2,1 -1,3 0,5 -1,1 2,0 0,2 m -15,-234 0,-2 -1,0 2,0 1,2 -1,-1 -1,1",
                        "MB": "m 403,590 20,1 5,0 49,1 -1,1 1,4 0,3 0,1 0,1 1,2 0,1 0,1 0,-1 -1,2 0,4 -1,0 -1,1 2,-1 0,1 1,1 0,3 0,-1 1,0 1,1 1,2 1,0 1,-1 0,-1 0,2 0,1 0,1 -1,1 1,3 -1,1 0,3 1,-3 0,-4 1,-3 0,-1 7,0 0,1 1,0 1,-1 1,0 1,5 2,6 1,1 1,2 1,4 -1,1 1,0 1,3 1,2 1,2 1,2 0,3 -2,3 -1,3 -2,0 2,0 3,-3 2,0 -2,3 3,-2 12,-6 7,1 5,3 9,2 -1,5 -15,21 -17,23 -1,1 -1,2 -1,1 -7,9 -4,6 -18,18 1,67 0,3 0,1 0,8 -2,0 -15,0 -11,0 -10,0 -14,-1 -8,0 -8,-1 -16,-1 0,-3 -1,-1 1,-7 0,-7 1,-7 -1,0 1,-8 -1,0 1,-7 1,-21 -1,0 1,-7 0,-7 1,-15 -1,0 1,-13 0,-1 0,-7 0,-2 1,-5 -1,0 1,-7 -1,0 1,-7 -1,-1 1,-4 0,-1 0,-2 0,-8 1,-3 1,-13 0,-5 1,-5 1,-7 0,-4 0,-1 3,-37 0,-1",
                        "ON": "m 654,861 1,-1 0,2 1,1 3,0 1,1 0,-2 -2,0 4,-3 3,3 1,-3 2,-1 0,1 0,1 1,0 1,2 0,1 1,-4 1,1 -1,1 2,0 -1,4 -2,2 -2,0 2,-1 1,-3 -4,4 -10,-3 -7,0 0,-2 1,1 1,-1 1,0 1,-1 m 22,-111 11,44 1,3 6,21 -1,3 3,3 1,3 3,3 2,2 5,5 3,2 1,1 15,-1 2,0 4,2 1,1 2,1 3,1 1,-1 0,-1 1,0 1,2 2,0 1,3 1,0 4,1 2,-2 2,0 4,1 4,-4 8,-5 3,-1 4,0 0,5 3,2 -1,1 0,2 -3,3 -3,2 -4,4 -5,8 -2,4 -6,5 -3,1 -4,4 -1,0 2,-2 -2,2 0,-2 -7,4 0,1 3,-3 1,2 3,-3 0,3 1,-1 1,-1 1,-1 -2,4 3,-1 -3,3 -2,0 0,-1 -1,1 -1,-1 1,0 -3,1 -2,-1 1,0 -3,0 0,1 1,0 -2,0 -18,9 -3,5 -2,0 -1,2 -1,6 1,1 6,0 3,-3 1,4 1,0 0,2 2,1 -2,2 -5,1 -11,4 -3,5 5,0 -6,1 -6,0 -7,1 -5,7 -1,3 0,-1 -1,1 -2,1 -4,4 -1,2 0,2 -3,-2 -4,2 -4,-1 1,-5 8,-2 1,-1 -1,-3 -2,0 -1,0 1,-1 0,-1 1,-3 -1,-3 0,-2 4,-3 5,-7 0,-3 -3,-12 1,-3 0,-3 2,-2 1,-4 -3,-7 -1,0 -3,-3 0,-1 -2,0 0,-1 6,-1 0,2 1,1 1,2 2,0 0,1 2,-1 -1,4 1,-1 2,0 0,4 2,-3 2,-1 1,3 8,1 1,-2 0,-3 -3,-1 0,-1 2,-2 2,2 3,-1 -1,0 0,-2 -1,0 1,2 -1,-1 -1,-2 -2,0 0,-1 -3,-2 1,0 -2,-2 1,-2 -1,-1 -1,0 -1,1 1,0 -1,0 -2,-1 0,1 -2,-4 -1,1 -5,-7 -5,1 -1,1 -1,-2 -1,1 -3,1 -1,-1 -2,2 1,-1 -1,-1 2,-1 -2,2 2,-2 -2,-1 -1,2 0,1 -1,-1 1,-1 -6,1 -4,0 1,-1 -5,2 1,-2 -5,2 -13,0 -1,-1 -2,1 -1,-1 0,-3 -1,0 -1,0 -3,1 -1,1 -2,-2 2,-3 -1,-1 0,2 -1,-3 2,0 1,-2 -2,-1 -2,1 0,1 -3,-2 1,-5 1,-1 -1,-2 -6,-4 0,-1 0,-1 1,-2 -1,-1 1,-2 -10,2 -4,-1 -5,-4 -2,-7 -3,-4 -2,1 -2,-1 -2,2 -2,-1 -4,1 -2,-1 -2,0 0,-1 -1,1 -4,-1 -2,-2 -2,2 -1,-1 1,3 2,1 0,2 -1,2 -1,0 1,-1 -1,1 -1,-1 1,2 -1,1 -1,-1 0,2 -1,1 -1,-1 0,-1 3,-3 0,-2 -1,-1 -1,-1 -1,2 1,1 -1,1 0,1 -1,0 0,2 0,1 -1,3 -2,1 2,-6 -5,3 -2,2 1,1 -1,5 -3,2 -3,0 -2,1 -1,0 -1,-2 -1,0 -6,1 0,-1 -3,1 -2,-3 -2,1 -3,3 -4,1 0,-1 -2,0 0,-2 -1,0 -3,-1 -1,-2 -1,0 -2,0 -1,1 1,1 -1,1 -2,-3 0,-1 0,-1 -3,0 0,-1 1,0 0,-1 -2,1 -3,-2 -2,0 -3,0 -2,1 0,1 -4,1 -1,-1 0,-2 -5,0 -1,-1 -2,0 -2,0 -2,-1 0,-2 -2,-10 -4,-1 0,-1 0,-3 -1,-67 18,-18 4,-6 7,-9 1,-1 1,-2 1,-1 17,-23 15,-21 1,-5 2,1 2,-1 2,3 2,1 2,2 5,1 0,2 3,3 0,1 2,1 0,2 0,-1 3,0 0,1 4,0 1,1 5,1 3,1 4,0 4,2 3,3 3,1 0,1 -1,0 -1,4 0,1 0,1 2,-5 2,-2 5,1 3,-2 2,0 2,-1 2,1 1,0 1,0 1,2 -1,-2 -1,-1 6,1 2,-2 1,1 0,2 1,-1 2,-1 1,0 3,1 0,-1 0,-1 1,1 0,2 0,1 0,1 2,1 -1,8 0,3 1,3 1,1 1,1 3,5 0,4 2,6 -1,2 0,5 3,1 2,2 1,2 6,3 0,2 1,1 -1,0 -2,2 -1,0 0,2 1,-2 3,0 3,2 5,1 1,2 1,0 4,2 2,4 1,1 1,2 -1,1 -2,1 0,3 -2,0 -1,3 1,0 0,1 0,-3 2,-1 2,-3 1,-2 6,1 3,2 2,1 2,1 m -3,-10 2,6 1,4 -2,-1 -2,-3 0,-2 0,-2 1,-2",
                        "NU": "m 478,141 0,-1 1,0 -1,1 m 36,-37 0,-1 0,-1 0,1 0,1 m 114,229 0,1 -1,1 0,-1 1,-1 m -70,-214 -1,0 2,0 -1,0 m 159,265 1,0 0,1 -2,-1 1,0 m -193,-138 1,1 -1,1 0,-1 -1,-1 1,0 m -85,-33 1,1 -1,1 -1,0 1,-1 0,-1 m 159,108 0,1 -1,0 0,-1 1,-1 0,1 m -136,-55 0,2 -1,0 0,-1 1,-1 m -4,-26 1,1 -1,1 -1,-1 1,-1 m -32,23 -1,1 -1,-1 2,0 m 101,73 1,0 1,0 -1,1 -1,-1 m 32,45 1,0 1,2 -1,0 -1,-1 0,-1 m -89,-121 0,1 -1,0 0,-1 1,0 m -3,-24 1,1 0,1 -2,-2 1,0 m 2,114 0,2 -1,0 1,-1 -1,0 0,-1 1,0 m 39,58 1,-1 0,1 1,0 -2,0 m 180,-35 1,0 1,1 -1,1 -1,-2 m -141,-162 0,1 -1,1 0,-1 1,-1 m -121,24 0,1 -1,1 -1,-1 1,-1 1,0 m 304,160 2,1 -3,1 1,-2 m -231,-91 -2,-1 2,-1 1,0 -1,2 m -127,118 -1,1 -1,0 1,-1 1,-1 0,1 m 238,-95 1,1 -1,1 1,2 -1,-1 0,-1 0,-2 m 33,307 0,1 0,2 -1,-1 0,-1 1,-1 m -296,-224 -1,1 -1,-1 1,-1 2,0 -1,1 m 22,20 -1,1 -1,-1 0,-1 1,1 0,-1 1,1 m 85,5 0,1 -1,1 -1,0 0,-1 2,-1 m 17,-222 1,3 -1,0 -1,-2 0,-1 1,0 m 83,-25 2,1 0,1 -2,-1 0,-1 m -130,215 0,1 0,1 -1,0 0,-1 0,-1 1,0 m 143,-79 1,1 0,1 0,1 -1,-1 -1,-1 1,-1 m 3,87 0,1 -3,-1 1,0 2,0 m -208,6 1,1 0,1 -2,-1 1,0 0,-1 m 208,47 -1,0 -1,0 1,-1 1,0 0,1 m 87,6 1,2 -1,0 0,-2 m -202,-80 1,1 -1,0 -1,-1 0,-1 1,1 m -101,17 -3,0 3,-1 0,1 m 391,65 1,0 0,1 -1,1 -1,0 0,-2 1,0 m -192,-88 -1,1 0,1 -2,-1 0,-1 1,0 2,0 m 18,-60 1,1 -1,1 -1,-1 0,-1 1,0 m 167,123 1,-1 1,1 -1,1 1,0 -1,0 -1,-1 m -74,-77 6,0 -2,1 -4,0 0,-1 m -268,-62 -2,0 -1,-1 1,-1 2,1 0,1 m 103,-100 0,1 0,1 0,1 0,1 -1,0 0,-1 0,-1 1,0 -1,-1 0,-1 1,0 m 160,496 0,1 1,0 -1,1 -1,0 -1,-1 0,-1 2,0 m -167,-149 0,-1 -1,0 0,-1 2,0 0,1 -1,1 m 204,-145 2,1 0,1 -1,0 -1,-1 1,0 -2,-1 1,0 m -323,28 2,2 -2,0 -1,-2 1,0 m 372,52 1,1 -2,0 0,-2 1,0 -1,1 1,0 m -187,-107 2,0 1,1 -1,0 -1,0 -1,0 0,-1 m 106,-30 1,0 -1,1 0,1 -2,0 1,-1 0,-1 1,0 m -51,-25 1,-1 1,1 -1,2 -1,0 -1,-1 1,-1 m -248,92 1,1 -1,0 1,1 -1,0 -2,-2 1,0 1,0 m 82,-173 0,1 -1,-1 1,0 -3,-1 1,0 2,1 m 281,194 1,0 -1,1 2,-1 0,2 -1,0 -2,-1 1,-1 m 13,9 2,1 0,1 1,1 -2,-2 -1,0 0,-1 m -253,-215 1,0 1,1 -1,1 -2,0 0,-1 1,-1 m 153,385 0,1 -1,0 0,2 -1,-1 1,0 0,-1 -1,0 1,-2 1,1 m -207,-408 0,1 0,1 -1,1 -1,0 0,-1 1,-2 1,0 m 62,181 -1,-1 1,-1 1,-1 -1,1 1,1 -1,1 m 125,-3 1,0 0,1 1,1 -2,0 -1,-1 1,-1 m 91,-8 -3,1 1,-2 1,1 0,-1 1,0 0,1 m -271,-111 1,-1 -3,1 -1,0 4,-1 0,1 -1,0 m -70,157 0,2 -1,0 0,1 0,-1 0,-1 0,-2 1,1 m 138,-16 0,-1 1,-1 0,1 0,-1 0,1 1,0 -1,3 0,-1 -1,0 1,-1 -1,0 m 134,221 1,2 0,1 -1,-2 -1,0 0,-1 1,0 m -175,-472 0,1 -1,0 -3,-1 2,-1 2,1 m -8,167 0,1 0,1 0,2 -2,-1 0,-1 1,0 0,-1 1,-1 m 245,49 2,2 -1,1 -1,-1 0,-2 m -78,198 1,0 -1,1 -1,2 0,-1 -1,1 0,-2 1,-1 1,0 m 109,-170 1,0 0,2 -1,-1 0,1 0,-1 -1,1 0,-1 -1,0 0,-1 1,0 1,0 m -365,27 0,2 0,1 -2,-2 0,-1 1,1 0,-1 1,-1 0,1 m 256,56 -1,2 -2,0 0,-1 1,-1 2,0 m -113,-264 1,0 1,2 -1,1 0,-1 -2,0 1,-2 m -80,181 1,1 0,2 0,1 -1,0 -1,-2 0,-1 1,-1 m 302,50 1,1 1,2 -1,-1 -1,-1 -1,-1 1,1 -1,-1 1,0 m -165,0 0,1 -1,0 -1,-1 -1,-1 0,-1 1,1 1,1 1,-1 0,1 m -67,-52 0,-1 -1,1 0,-1 0,-1 0,-1 0,2 -1,-1 1,-1 1,0 1,1 0,1 -1,1 m 180,-27 -3,0 1,0 0,-1 1,0 0,-1 1,-1 0,2 0,1 m -217,-85 1,1 1,2 -2,0 -2,-1 2,-2 m -10,1 0,2 -2,1 -1,-1 1,-1 1,-1 1,0 m 261,92 1,-1 -1,-1 1,-1 0,-1 1,-1 0,1 0,2 0,1 -1,0 0,1 -1,0 m -212,14 1,2 -1,2 -2,-2 0,-1 1,-1 1,0 0,1 0,-1 m 154,-46 1,1 0,1 0,1 -3,0 1,-2 2,0 -2,-1 1,0 m -69,54 1,-1 2,2 -1,2 -1,0 -1,0 0,-2 0,-1 m 34,-13 1,0 2,2 0,2 -1,0 0,-1 -2,-1 0,-1 0,-1 m 37,111 1,1 3,-1 1,1 -2,1 -3,0 -2,-1 2,-1 m -234,-107 0,1 -1,1 -1,1 0,-1 0,-1 1,0 -1,-2 0,-1 1,0 0,1 1,1 m 229,72 1,0 -1,-1 1,0 -1,0 0,-1 1,-1 0,2 1,1 -1,0 1,1 0,1 -1,0 -1,-2 m -18,83 -2,2 -2,2 0,-1 0,-1 4,-2 m -167,-289 1,2 0,2 0,1 -2,-1 0,-2 -1,-1 2,-1 m 121,140 -1,0 0,-1 1,-1 0,-1 -2,-1 1,0 2,0 1,2 -2,0 0,1 1,0 -1,1 m -159,-29 2,1 -2,0 1,1 0,2 -3,-3 0,-1 0,-1 2,1 m 289,62 1,1 -1,1 0,1 -1,0 1,0 -2,1 0,-1 1,-3 -1,-1 1,0 0,2 0,-2 1,1 m 27,49 1,1 0,-1 1,1 2,1 -1,1 -3,0 -1,-1 0,-1 0,-1 1,0 m -290,-163 0,1 1,2 -1,2 -1,0 -1,-1 -1,-1 0,-1 1,-1 2,-1 m -61,91 2,3 0,1 -1,0 -3,-2 -1,-2 1,0 2,0 m 107,-220 3,0 0,1 0,1 -4,2 -1,0 1,-1 0,-2 1,-1 m 108,142 0,-1 -2,0 0,-1 -1,-1 0,-1 0,-1 1,0 0,2 1,1 0,-3 2,1 -1,2 0,1 1,0 -1,1 m 122,112 0,1 1,0 0,1 1,1 0,-1 1,1 0,1 -3,-1 -1,1 -1,-1 0,-1 1,0 -1,0 1,-1 0,-1 0,1 1,-1 m 14,24 1,1 -1,0 1,0 -1,1 0,1 0,1 -3,0 -1,0 1,-1 -1,-1 0,-1 2,0 1,0 -1,-1 1,0 1,1 0,-1 m -77,256 1,0 -1,1 0,1 0,1 -3,1 -1,-1 0,-1 1,0 2,-2 1,0 m -210,-329 1,1 -1,1 1,1 -1,1 -1,-1 0,-2 -2,0 -1,-2 3,0 0,1 1,-1 0,1 m 63,-16 1,-1 -1,0 1,0 2,1 1,-1 2,2 -2,1 -2,0 -1,0 -2,-2 1,0 m -83,-136 -5,3 -2,0 5,-4 2,1 m -10,162 0,1 1,1 -1,1 0,1 0,1 -1,-1 0,2 -1,-1 0,-1 -1,1 0,-1 0,-3 1,0 -1,-1 1,-1 2,1 m -11,4 0,2 -1,0 0,1 0,1 -4,-3 2,-1 1,-2 2,2 m 10,-153 0,1 0,1 -6,1 -2,1 -1,-1 0,-1 6,-2 3,0 m 192,78 3,0 0,2 -1,0 -1,2 0,1 -1,0 -1,-1 -1,-1 1,0 0,-3 0,-1 1,1 m -156,60 1,1 0,3 -1,1 -2,0 1,0 -1,-1 1,0 0,-2 0,-1 -1,-1 0,1 0,1 -1,2 0,1 -1,0 0,-1 1,-1 -1,-1 1,-1 0,-1 1,-1 1,0 1,1 m -29,-176 2,1 1,2 0,1 -1,0 -2,-1 -1,1 -2,0 -1,-1 0,-1 0,-1 0,-1 4,0 m 112,28 1,0 0,1 -1,3 2,1 -3,0 0,1 0,1 -1,0 0,-1 -1,0 1,-1 0,-1 1,-2 0,-2 1,0 m 186,273 2,1 0,2 -2,4 -2,1 0,-1 0,-6 2,-1 m -304,-260 -2,1 0,2 -1,1 -3,0 -1,-1 1,0 -1,-1 2,0 -1,0 1,-1 2,-1 3,-1 0,1 m 31,9 1,1 -1,1 0,1 -2,-1 1,-1 -1,-1 0,2 0,1 -1,1 -1,1 -1,0 0,-1 0,-2 3,-2 0,-1 2,1 m 135,109 2,1 1,-1 1,0 0,1 -1,3 0,1 1,0 0,1 -1,0 -1,1 -1,-1 -1,-1 -1,-2 0,-1 1,-2 m -105,-137 1,1 1,1 0,1 -1,1 0,1 0,2 1,0 -1,1 0,-1 0,1 -1,0 -1,-1 -2,-4 0,-2 3,-1 m 97,133 1,1 0,1 0,1 -3,3 -1,0 -1,1 1,-4 1,0 0,-1 1,0 0,-1 1,-1 m 28,15 1,2 2,1 -1,1 1,0 0,1 0,1 1,0 -2,0 0,-1 -2,0 -2,-1 0,-2 0,-2 2,0 m 121,60 2,2 0,1 1,0 -1,0 3,1 1,1 -2,0 1,0 -1,0 -3,-2 -2,-2 0,-2 1,1 m -79,219 -7,5 1,-1 -1,0 3,-2 4,-2 1,0 -1,0 m -23,-174 1,0 -1,1 1,0 -1,1 -2,0 -4,-2 -1,0 -1,-1 1,0 -1,0 1,-1 2,0 0,-1 5,2 0,1 m -207,-241 0,1 0,1 0,1 -1,0 0,1 -3,1 -4,-1 1,0 -3,-1 0,-1 1,-2 3,0 4,-1 2,1 m 22,-72 0,-1 -2,-1 1,-1 -1,0 1,-1 -1,0 -1,-1 -2,0 0,-1 1,-1 -1,-2 1,-1 1,-1 1,1 2,-1 0,2 1,2 0,2 0,2 -1,3 m 304,306 -1,1 1,0 0,1 0,1 0,1 0,1 0,1 -1,0 0,-1 -1,0 -1,0 -1,-1 -2,0 -2,0 0,-2 3,-2 3,-1 0,1 1,0 1,0 m -201,-26 1,0 0,1 1,1 1,3 1,0 0,1 0,1 0,1 -1,0 -1,0 0,-2 -3,-1 0,-3 -1,-1 0,-1 1,-1 1,1 m -132,-217 1,1 -1,1 1,0 1,-1 1,1 0,1 1,1 1,1 -1,2 -1,1 -5,-1 1,-1 -1,0 0,-1 0,-1 0,-1 -1,-1 1,-1 -1,0 3,-1 m 271,245 2,0 0,1 1,0 0,1 1,-1 1,1 -1,2 -3,1 -1,-1 -1,-2 -1,0 1,-1 -3,0 -1,0 1,0 -2,-1 4,-1 2,1 m -158,-70 1,2 0,2 0,2 0,3 -1,0 -2,-1 -1,-3 1,-1 0,-1 -2,-1 1,-1 0,-1 1,-3 2,1 0,1 0,1 m -116,-149 2,2 0,1 0,1 0,2 -1,1 -3,2 -2,-2 -1,-1 2,-3 -1,0 1,-2 1,-1 2,0 m 34,31 0,1 -1,1 -1,1 -1,1 -2,1 -1,0 0,-1 -1,1 -1,1 -1,-1 2,0 -3,-1 -1,-1 1,0 2,-1 1,-1 2,0 3,-2 2,1 m 37,-75 2,2 0,2 -1,3 -2,1 0,-1 -4,-3 0,-1 0,-2 0,-2 1,-1 3,1 1,1 m 87,151 1,-1 1,0 2,1 -2,3 -2,0 -1,0 1,2 0,1 -2,-1 0,-2 -1,2 0,-1 -1,-1 -3,0 1,-1 2,0 0,1 0,-1 -1,-1 1,-1 0,1 1,-1 1,1 1,0 1,-1 m 11,5 2,0 1,1 -1,3 -1,-1 0,1 0,1 0,1 0,2 0,1 -1,1 -2,2 -2,0 0,-1 0,-1 1,-2 1,-1 1,-1 0,-1 0,-2 1,0 -1,-1 0,-1 1,-1 m 34,116 4,-1 0,1 1,0 1,0 0,1 2,1 -1,2 1,0 -3,3 -7,-5 -1,0 3,-2 m -204,-278 3,4 0,2 -1,0 0,1 1,0 1,1 1,2 -1,0 1,1 0,1 -1,2 -1,0 -2,-2 -1,1 0,-1 0,-1 -1,-2 1,-1 -1,0 0,-1 0,-1 0,-1 0,-1 0,-1 -1,-3 1,-1 1,1 m 213,178 2,0 0,2 1,1 1,0 -1,2 -8,2 -2,-2 -1,-1 0,-1 0,-1 2,-1 0,-2 1,0 0,2 3,-1 1,1 1,-1 m -156,-174 1,1 -1,0 -1,1 1,0 -1,1 0,1 0,1 -6,0 -5,0 -1,-1 -1,-2 0,-1 0,-1 4,-2 0,1 1,0 2,-1 2,1 1,-1 1,1 1,-1 2,2 m 87,235 3,2 0,1 -1,1 3,1 1,0 1,0 0,1 0,1 -1,0 -1,-1 -1,0 0,1 -1,0 -1,0 0,-2 1,0 -1,0 0,-1 -1,1 -2,0 1,-1 -1,-2 -1,0 0,-1 -1,0 -1,-1 -1,-2 4,1 1,1 m 82,250 5,4 2,3 0,1 -4,0 -3,-1 -9,-1 -2,-1 1,-2 1,-1 5,-2 4,0 m -25,-175 0,1 0,3 -1,1 -1,0 -1,-2 -4,-1 0,-2 -1,-2 0,-2 0,-3 1,-3 2,-1 1,0 1,-1 1,1 1,1 1,1 1,1 0,3 0,1 -1,4 m -213,-213 -1,0 0,1 1,1 -1,0 0,-2 -3,-4 1,-1 -2,-1 1,0 -1,-2 1,0 -2,-3 -3,-2 3,-4 3,0 6,0 1,3 2,1 0,1 -1,2 0,1 0,1 0,1 -1,2 0,1 -2,1 0,2 -2,1 m 48,-121 2,0 2,1 1,1 1,1 1,1 1,0 -1,2 2,1 1,-1 3,3 1,1 -3,2 1,1 0,1 1,1 0,1 -1,3 -1,-1 -4,3 -1,-1 -1,0 1,0 0,1 -1,0 0,1 -2,0 0,-1 0,-1 1,-1 -4,-2 1,-1 0,-1 3,1 1,-1 -1,0 1,0 -2,-2 -3,0 0,-1 1,0 -2,-1 1,-1 -1,-1 1,0 -2,-1 0,-1 0,-1 2,0 -2,-3 1,-1 -1,0 1,-1 -1,0 1,-1 1,0 m 135,317 3,0 0,1 0,1 1,2 -2,3 -2,1 0,2 -4,6 -1,1 -1,0 -1,-1 -3,3 -1,-3 -3,-2 1,-1 1,-3 1,-2 0,-3 1,-2 1,0 1,1 1,0 3,-3 2,0 1,-1 1,0 m -126,-232 -1,-1 0,-1 0,-1 0,1 0,-1 0,-1 -1,3 -1,-1 0,-1 1,0 -2,0 0,-1 0,-1 1,-2 0,-1 2,0 0,-1 1,0 -2,0 2,-2 1,0 -2,-1 0,1 -1,0 1,-1 1,-1 1,1 -1,-1 1,-1 1,-1 1,0 0,-1 1,0 1,-1 3,1 2,3 2,2 0,1 -1,0 2,1 0,1 0,2 0,1 -1,1 1,0 1,1 0,2 0,3 -1,0 1,0 -1,1 -1,1 -3,0 -2,1 -1,-2 -1,1 0,-1 0,-1 -2,-1 0,1 0,-1 -1,0 0,1 0,-1 -1,0 0,-1 0,1 -1,0 0,-1 m 158,114 0,2 -1,2 2,2 0,1 1,2 1,1 0,2 -1,4 -1,1 -1,1 -2,2 -2,0 -3,2 -3,0 -2,-3 -1,0 0,-2 -2,-3 0,-3 1,-7 1,-3 2,-2 0,1 2,0 1,-2 1,0 5,1 2,1 m -70,-101 2,-1 2,1 1,-1 3,1 8,-2 4,1 2,0 2,2 0,1 1,1 1,-1 0,1 1,1 0,1 1,1 2,0 0,1 0,1 1,0 -1,1 2,0 0,2 -2,1 -7,0 -4,1 -5,5 -2,0 -3,-1 -2,-4 0,-1 0,-1 0,-1 -1,0 -4,-1 0,-1 -1,0 0,-1 0,-1 1,-1 -1,-1 -1,-1 1,-1 -1,-1 0,-1 m -123,-96 1,-1 1,0 0,-1 1,0 0,-1 -1,0 -1,1 1,-2 -1,0 0,1 -1,0 0,-1 2,-2 -2,-2 0,1 -1,-1 -1,2 -1,1 -1,0 1,-1 1,-3 -1,-1 -2,1 0,-1 0,-3 0,-1 1,-1 0,-1 1,1 5,-1 1,0 3,3 0,1 0,1 1,1 0,3 -1,0 1,0 0,1 0,1 1,-1 0,-1 0,-1 2,-1 1,0 2,3 0,-1 1,1 -2,3 1,1 3,-1 0,1 2,2 -1,1 0,1 1,-1 0,1 1,0 0,1 0,1 -1,1 0,1 0,1 0,2 1,0 0,2 2,3 0,1 -1,1 1,2 -1,1 -3,0 0,1 -4,-2 1,-1 -1,-3 0,-1 -1,-1 0,-2 -1,1 -1,-2 -3,-1 -2,1 -1,-1 0,-1 1,-1 -4,1 -1,0 0,1 -1,0 -2,-1 -1,-2 -1,-2 0,-1 1,-1 1,-1 5,3 0,-1 0,-1 1,0 -1,0 1,-1 -3,0 m 8,203 1,-2 1,1 0,-1 2,1 1,-1 2,-1 0,-1 0,-1 1,1 1,-1 -1,-3 1,0 -1,-1 0,-2 1,1 1,2 1,0 0,1 1,0 -2,-3 -1,-1 1,-3 2,-3 4,3 1,1 -1,1 0,1 1,-2 3,5 1,0 2,1 2,2 0,1 0,3 0,2 1,0 -1,-1 0,-1 0,-1 1,-1 1,3 0,1 0,1 1,2 1,1 2,-1 1,1 -2,4 0,-1 0,-1 -1,0 -1,0 0,1 -1,1 1,1 -2,0 -2,3 -1,0 -4,-1 -1,-2 0,1 0,1 -3,-1 -2,-2 -2,-1 -2,-1 1,-2 -1,-1 -1,1 1,1 -1,1 -1,-1 -2,-1 0,-1 1,-1 -1,0 -1,-1 0,1 -1,0 1,1 -1,1 -2,-2 0,-1 -1,-1 0,-1 m 9,-144 -3,0 0,-1 0,-1 -1,0 0,-1 0,-1 1,0 1,1 2,-1 -2,0 -1,-1 -2,-1 0,-1 2,0 -1,-1 1,1 1,0 -1,0 1,-1 -2,-1 1,0 -3,0 -1,-3 1,-1 0,-1 2,-1 2,1 1,-1 1,1 -1,0 1,0 1,2 1,0 -1,1 1,2 0,-2 1,1 0,-1 -1,-1 0,-2 1,0 1,1 0,-1 1,0 0,-1 -1,0 -1,0 2,0 1,0 -1,1 2,1 -1,0 3,2 0,1 0,1 -1,2 2,2 0,1 -1,2 0,2 0,2 0,2 -2,1 3,1 0,1 0,1 0,2 0,1 1,1 -1,0 -1,-1 1,-1 -2,-1 0,1 0,1 0,-1 -1,0 0,1 -1,0 2,1 -1,0 0,1 -1,0 0,1 0,1 1,-1 0,1 0,-1 0,1 1,1 0,1 1,1 -2,0 -1,-1 0,-1 0,1 0,1 1,0 0,1 0,1 -5,1 0,-1 0,-1 -1,0 0,1 -2,1 0,-1 0,-1 1,-1 -2,2 1,0 -1,1 -4,-1 -1,-1 1,-1 -1,-1 1,0 -1,-1 3,-1 -2,-1 1,-1 -2,1 -1,-2 2,0 -2,-1 4,-1 -2,0 2,0 -1,-1 2,0 -1,-1 2,0 0,-1 -1,0 0,-1 4,0 1,0 0,-1 -9,1 -4,1 -1,0 -6,1 0,-1 -1,-1 1,0 0,-1 1,-1 3,0 -1,-1 1,-1 -2,0 1,-1 2,-1 2,2 0,1 1,1 2,-1 -3,0 1,-1 -1,-1 -1,-1 2,-2 -2,1 -1,-1 0,-2 1,-2 2,-1 -4,0 1,0 0,-1 -1,0 1,-1 0,-1 1,-1 3,1 1,2 -1,1 2,2 1,1 -1,1 2,-1 -1,1 1,1 1,2 0,1 1,0 0,1 3,-2 m 20,57 0,-1 0,-1 -1,-2 1,-1 -1,0 1,0 -1,-1 1,-1 0,1 -1,-2 0,-1 0,-1 3,0 2,2 2,0 -2,-1 -1,-1 0,-1 0,-1 -1,1 -1,-2 0,-1 1,-1 3,-2 3,-1 1,0 1,1 1,0 -1,0 0,-2 2,0 4,1 0,1 1,-1 2,2 0,1 1,0 0,-1 1,0 2,-1 3,1 4,1 1,1 0,1 0,-1 -1,0 0,2 -1,5 -2,0 1,2 -2,3 0,1 -2,1 1,1 -1,4 -1,3 -1,2 -2,1 -5,-1 -1,-1 -2,1 -4,1 0,1 1,-1 1,1 1,-1 0,2 2,2 0,2 -1,2 -1,1 -1,3 0,2 -1,1 1,-1 0,1 0,1 -3,-1 -4,1 0,-1 0,-1 0,-1 2,-1 -2,1 0,-1 -1,-1 1,-3 -1,-1 1,0 -1,-1 1,0 -1,-1 0,-1 0,-1 -1,-1 -1,-1 0,-1 0,-1 -1,0 1,-2 0,-1 -1,-1 1,-1 -1,-1 1,-1 0,-1 0,-1 0,-1 m -33,-2 -1,-1 0,-1 -2,-1 0,-1 1,-1 1,0 1,-1 1,0 2,3 0,1 1,0 -1,-2 0,-1 0,-1 -2,0 1,-1 -1,0 0,-1 -1,-1 1,0 0,-1 1,-1 1,1 0,-1 2,0 1,1 1,2 0,-1 0,-1 1,-1 0,-1 0,1 -1,0 0,1 -1,0 -1,-1 1,-1 2,-1 0,1 0,1 2,2 2,1 -1,1 1,0 0,-1 2,0 1,-1 3,0 1,-1 0,-1 1,0 0,1 1,-1 1,1 1,0 2,2 0,1 0,2 -2,1 -1,0 0,1 -1,-1 -1,2 1,0 1,-1 1,1 0,1 0,1 -4,2 0,1 -2,3 -2,1 0,1 1,1 -1,1 2,-2 2,-1 4,2 0,1 -1,1 1,0 0,1 1,1 0,1 -1,1 0,1 1,0 1,-1 2,-2 0,1 1,1 0,1 0,1 0,1 0,1 1,1 -1,0 0,1 -2,1 -1,0 2,1 0,1 0,2 0,1 -2,1 1,0 1,0 0,2 -2,0 2,0 0,2 -1,1 0,-1 -1,0 -1,1 -1,2 -2,2 -4,-1 -1,0 -1,-2 0,-1 1,-2 0,-1 0,1 -1,0 -1,2 0,1 1,1 0,1 1,1 0,1 1,0 -1,3 -2,2 -2,1 0,-1 -1,-2 -2,1 0,-3 -1,-1 0,-1 -1,-1 -1,-3 0,-1 -1,-1 -1,-1 -2,-3 0,-1 0,-1 -1,-1 -2,-1 0,1 -1,-1 1,-1 -1,0 -1,-2 -1,1 0,1 -1,0 1,0 -1,-1 -1,0 -1,-2 1,0 -1,0 0,-2 -4,-4 0,-2 0,-1 1,-1 0,-3 1,-1 1,-1 2,0 0,1 1,1 0,1 0,1 1,1 0,-1 0,1 1,1 0,1 1,0 0,1 -1,0 3,1 1,0 -1,-1 1,0 2,0 1,0 0,-2 -1,0 0,-1 0,-1 0,-2 1,0 -1,1 1,0 -1,0 1,1 0,1 0,1 1,-1 1,-1 -1,-1 0,-1 0,-1 -1,0 1,-1 -2,1 0,-1 -1,0 1,-1 0,-1 1,-1 1,1 1,0 -1,0 1,1 2,0 -2,-1 -2,-3 0,1 -1,1 -2,0 -1,-1 m 36,-167 3,1 1,-1 0,-1 0,-1 -2,0 -2,-1 0,-1 0,-1 1,-1 1,1 1,1 0,-1 0,-1 5,3 1,2 0,1 1,2 0,1 2,3 0,1 2,2 0,1 0,1 2,1 1,1 0,-1 1,-1 1,1 1,1 -1,2 1,0 0,1 0,1 1,2 2,2 0,-1 1,1 -1,-2 1,0 -1,-1 -1,-1 0,-1 0,-1 0,-1 0,-1 3,1 1,2 0,1 0,1 0,1 -1,1 0,2 3,-1 1,2 0,1 -2,2 1,0 1,-2 0,3 0,1 0,2 -1,2 0,1 1,0 1,-1 0,-1 1,0 0,1 0,-1 0,-1 2,0 1,1 0,1 0,1 0,1 1,-1 0,-2 0,-1 1,1 0,1 1,1 2,2 1,2 -2,2 -4,3 0,1 0,1 -1,1 -1,-2 0,1 0,2 -1,3 -1,3 -1,-1 0,-1 -1,-1 0,-2 0,-2 1,-1 -1,0 1,-1 -1,1 -1,1 0,2 0,2 1,3 -1,0 2,0 0,2 0,2 -1,1 -2,-3 -1,0 1,0 0,1 1,3 0,2 0,2 -1,1 -1,0 -1,-3 -2,-2 -2,-4 0,1 1,2 0,1 1,1 2,5 -1,0 -1,-2 -1,1 0,-2 -1,-1 -2,1 1,1 1,2 -1,1 -5,-1 -1,-1 -1,0 -1,-1 0,-1 -1,0 -1,0 0,-1 -1,-1 0,-1 4,-2 1,0 -4,-1 -1,-1 -2,2 -1,-2 1,0 -1,0 -1,-2 1,-1 2,1 -2,-2 -2,0 -1,-3 1,-1 1,-1 1,-1 0,-1 2,0 1,0 2,0 2,-1 5,-1 0,-1 -7,2 -1,-1 -1,0 0,-1 3,0 3,-2 0,-1 -4,2 -2,-1 2,-1 -1,-1 -1,0 -1,1 -1,-1 0,2 -1,1 1,-2 -1,0 0,2 -1,0 0,1 -1,1 -1,0 0,-1 0,-1 1,-1 -1,0 -1,-1 -1,1 1,1 -1,0 -2,2 0,-1 -1,-1 1,0 -1,0 0,-1 0,1 -2,-1 0,-1 1,0 -1,-1 1,0 -1,-1 5,-2 1,-1 0,-1 0,-1 -2,2 -4,1 -1,-3 -1,1 0,-1 0,-1 0,-1 -1,0 0,-1 1,0 1,0 -2,-1 0,-2 1,0 -1,0 -1,-1 1,-2 1,1 2,2 4,0 2,1 -2,-2 1,-1 2,-1 -2,-1 -1,1 -2,1 -1,-1 1,-1 0,-1 -1,1 0,-1 -1,1 -3,-1 1,-1 -1,-1 2,0 -1,0 1,-1 3,1 -1,-1 -2,-2 1,-1 -1,-1 4,-1 1,1 3,0 0,1 1,0 -1,-1 -2,-1 -1,-1 0,-1 2,0 -3,-1 1,-1 -1,0 -1,0 -1,0 1,-2 -1,0 1,-1 0,-2 1,-1 1,1 1,0 0,1 0,-1 2,0 -1,-1 1,0 -1,0 -1,-1 m 78,342 1,-5 0,-1 0,-1 -1,-2 1,-2 -1,-3 0,-2 0,-3 0,-2 0,-2 1,-3 2,-2 1,0 0,2 1,0 2,0 0,1 0,1 1,2 -2,1 3,2 1,2 1,2 1,0 0,-3 1,-2 1,0 0,1 1,0 2,1 -1,1 1,1 3,0 -1,1 1,0 3,-1 1,1 1,1 0,1 1,1 2,0 2,1 0,1 1,1 2,0 1,-1 2,1 0,1 1,-1 4,3 0,3 0,1 2,2 1,2 -3,2 0,1 4,-2 0,-1 1,0 4,1 0,-1 -1,0 0,-2 2,1 1,1 2,0 0,1 0,1 -2,0 1,1 4,0 1,1 -3,1 0,2 -3,3 0,2 -1,0 -4,-2 -1,1 -2,-1 -1,0 -2,-1 -1,1 -1,0 -1,0 -1,-1 1,-2 0,-2 -2,-1 -3,1 -2,1 0,-1 1,-3 -1,-1 -1,1 0,1 -2,0 -1,0 0,1 -1,2 1,1 0,1 0,1 0,1 -1,0 -2,4 -2,0 -1,2 0,3 -1,2 -5,5 -2,0 -1,-1 -1,-1 -1,-3 1,-2 -1,-1 0,-2 0,-1 -1,0 0,-1 -1,1 -4,2 -3,0 -2,2 -2,1 -1,-1 0,-3 1,-4 6,-4 0,-2 -1,-1 -1,-1 0,-1 0,-1 0,-1 0,-2 -1,-1 m -87,-252 1,1 1,-1 0,-1 3,0 1,0 3,2 2,0 0,1 1,0 1,0 0,-1 2,1 0,1 2,2 0,2 0,1 0,2 -2,2 1,1 0,-1 1,-1 1,-2 0,-1 3,0 2,-1 3,-1 2,1 1,1 1,0 1,1 0,1 -1,1 -2,-1 -2,0 0,1 8,1 3,1 1,2 -1,1 -3,0 -2,1 2,0 -6,-1 -2,-1 2,1 -1,2 1,-1 3,1 -1,1 1,-1 2,1 0,1 -5,0 2,0 -1,1 1,1 -1,0 0,2 0,-1 1,-1 0,-1 1,0 1,0 0,1 0,1 1,-2 1,1 0,-2 2,2 0,2 1,-1 2,1 0,4 -2,0 -1,1 2,0 -1,0 2,-1 1,3 1,0 0,1 1,-1 -1,-3 1,-1 0,-1 3,3 -1,1 1,0 1,-1 1,-1 1,0 0,1 1,1 -1,0 1,0 0,-1 -1,-1 2,-2 2,3 1,0 2,1 -1,1 2,-1 3,0 -1,-1 -2,-1 3,-2 -1,0 1,0 1,0 0,-1 1,0 0,-1 3,0 -1,0 0,-1 2,-1 0,-1 1,0 0,-1 1,-1 0,1 1,0 1,1 2,0 2,-3 1,0 7,-1 0,1 -1,1 0,1 5,0 2,0 0,1 1,0 -1,1 -1,1 2,-1 2,0 0,1 1,1 0,1 1,1 -1,2 -1,0 0,1 0,1 -2,2 1,1 0,1 1,-1 0,-1 1,-1 1,0 1,0 0,1 1,1 -2,3 -2,-1 0,-1 0,1 -1,-1 0,1 2,1 0,1 -1,0 1,1 0,1 0,1 0,1 -5,2 -3,3 -4,-1 1,0 -1,-1 0,1 -1,0 -1,0 -2,-2 0,-1 0,-1 0,-1 -1,0 -2,-2 0,1 0,1 1,1 0,2 0,2 0,1 -7,2 -1,-1 0,-2 0,-1 -1,1 1,1 0,2 -1,1 -1,-1 0,-1 -1,-2 0,3 0,1 -3,1 0,-2 -1,-1 0,2 -1,1 -1,-1 -1,-2 0,2 1,1 -2,1 0,-1 0,1 -2,0 0,-1 0,-1 -1,2 -1,0 0,-1 -1,1 -1,0 0,-1 -1,1 -2,0 0,-1 0,-1 1,-2 0,-1 -1,-1 1,0 -1,0 0,-1 -1,-2 0,1 1,1 -1,0 0,1 0,2 -1,1 0,-3 -1,-1 0,1 0,2 -1,-1 1,1 0,2 -4,2 -1,-1 0,1 -2,-1 0,-1 -1,1 -1,-1 0,-1 0,-1 -1,0 0,1 -1,0 1,-3 0,-1 -1,3 -1,0 1,3 -1,0 0,-1 0,-1 -1,0 0,1 -1,0 0,-1 -1,0 -1,0 1,0 -2,-1 0,-1 0,-1 0,-1 0,-1 -1,-1 0,-1 1,-1 0,-1 -1,0 -2,-1 1,-1 0,-2 0,-1 -1,0 1,-1 1,-3 1,-1 -1,0 0,-1 -1,-2 1,-1 0,-1 -3,-3 0,-2 -1,-1 0,-1 -2,-2 0,-1 -1,0 -2,1 0,-1 0,1 0,1 -5,0 0,-1 -2,2 -1,-1 0,-1 2,-1 0,1 0,-1 -2,0 -1,-1 1,0 -3,0 -1,-1 0,-1 0,-1 2,0 0,-1 0,1 -2,0 0,-1 -1,-1 -1,0 0,-1 -2,-1 0,-1 0,-1 1,1 2,0 -1,0 0,-1 1,0 -3,-1 0,-1 1,-1 -1,0 1,-1 m 81,-149 2,0 0,-1 0,1 0,-1 2,0 0,1 0,2 -3,4 0,2 3,-2 0,-2 1,0 0,-1 1,-3 1,0 0,1 2,0 -1,-1 0,-1 -1,0 1,-1 0,1 1,-1 0,1 1,1 1,-2 2,0 0,1 -1,1 1,0 1,1 1,0 0,1 1,0 0,1 1,1 0,1 0,-1 0,-1 1,0 -1,0 1,-1 1,0 0,1 0,-1 2,0 2,1 1,1 1,2 0,2 -1,1 0,3 0,1 -2,4 0,1 0,1 -1,0 1,1 0,1 -4,1 1,0 -1,1 -1,1 2,-1 -1,2 -1,0 0,1 -4,2 -1,2 -5,-2 4,2 -1,1 1,0 4,-3 2,0 -4,8 -1,1 -1,0 0,1 0,1 -2,1 2,0 -1,2 1,-3 9,-11 3,-3 1,1 0,1 0,1 -1,4 -1,1 -1,5 0,3 -1,1 0,1 -1,3 -1,4 0,1 0,1 1,3 -1,1 -2,1 -1,-1 0,-1 -3,-1 3,2 0,1 0,1 3,1 0,2 -3,0 -2,3 -2,-1 -2,0 3,2 -1,1 3,-1 2,-2 1,0 1,1 -1,1 0,1 -2,2 1,1 0,-1 1,0 1,1 -1,1 -3,4 -3,0 -1,-1 1,-1 -1,0 -3,1 0,-1 -1,0 -3,2 1,1 2,-1 3,0 1,1 1,1 0,1 1,1 -1,1 -1,1 -1,-1 0,1 0,1 0,1 -2,0 -2,-1 0,1 1,1 0,1 -2,0 -1,0 -1,0 -2,-1 -2,1 -1,-1 0,1 4,0 0,1 2,1 -4,1 -1,-1 -1,-1 -1,0 2,3 -3,0 -1,0 1,0 3,1 -1,1 3,-2 2,0 1,0 6,-1 -1,1 1,0 0,1 1,1 0,1 -4,0 -2,0 0,-1 -1,-1 -8,3 1,0 5,-1 3,0 0,1 -4,1 -1,2 -2,-1 -2,0 -2,1 4,-1 1,1 -1,1 0,4 0,1 1,-5 3,-3 5,0 0,1 -2,2 0,1 1,-1 3,-1 2,0 1,0 0,1 0,1 1,1 -1,0 0,1 1,0 1,1 -1,0 0,2 -2,1 -2,0 -2,0 -1,1 2,0 1,0 3,0 2,1 0,1 -1,1 0,2 -3,0 -3,1 0,1 1,0 5,0 0,1 0,3 -2,-1 -1,1 -1,2 -2,0 -3,1 -2,-1 1,0 -1,2 3,2 0,1 0,1 1,0 1,1 -1,2 1,1 -1,2 0,1 -2,2 0,-1 0,-1 -2,3 -2,0 -2,1 -3,0 -3,-2 -1,-1 -1,0 0,-1 0,-1 -2,0 0,-1 0,-1 0,1 0,1 2,1 0,1 0,1 0,1 1,1 2,-1 0,1 -3,2 -1,-1 -1,1 2,1 0,1 1,0 3,-3 4,1 0,2 0,1 1,-3 1,-2 2,0 2,1 0,1 0,1 -1,3 0,1 3,-1 0,2 1,0 0,-1 0,-1 0,-1 1,0 0,-1 1,-1 1,1 0,1 1,0 0,1 1,1 0,2 0,1 -1,1 0,2 0,1 0,1 -2,0 0,-1 0,-1 -1,0 0,1 0,2 -1,1 0,1 0,1 -1,1 -2,2 -1,0 -2,2 -1,0 -1,1 0,-1 0,-1 0,-3 0,-1 -1,-1 -1,0 0,-1 -1,-1 -1,2 -2,0 -1,0 1,-1 -1,-1 1,-2 -1,2 -1,0 -1,-1 -1,-2 -1,-1 1,2 0,1 1,1 2,1 0,1 1,2 -1,0 -3,2 -1,-2 -1,-2 -2,-2 0,-1 0,-1 0,2 1,2 1,1 0,1 0,1 0,1 -2,-1 -1,0 -1,-1 -1,-2 -1,-1 0,1 1,0 0,2 0,1 0,1 -2,0 -1,0 0,-1 -1,-1 0,1 1,2 3,1 0,1 -4,1 0,-1 -1,1 -2,-1 -3,0 -1,-1 0,-1 1,-2 -2,-1 2,2 -2,1 0,1 0,2 -2,-1 0,-1 -2,0 0,-1 0,-1 0,-1 -1,0 0,1 1,1 0,1 0,1 0,1 1,0 -1,0 -1,0 -1,-1 0,1 -2,0 -1,0 1,-1 -1,-1 1,0 0,-1 -2,-2 1,-1 -1,-1 0,-1 0,-1 -1,2 1,2 0,1 1,1 0,1 -1,1 1,1 -1,0 0,-1 0,-1 0,-1 0,-1 -1,0 0,1 0,1 1,1 -1,0 0,1 -1,0 0,-1 -1,1 -1,-2 0,-1 -1,0 0,-1 1,-2 0,-1 -1,0 0,-2 2,-3 1,-1 2,-1 0,-1 -1,0 1,-1 3,0 1,0 1,0 -1,0 1,-1 1,-1 1,1 1,-1 -3,0 0,-1 1,0 0,-1 -1,0 1,-2 -2,1 -2,0 0,-1 0,-1 0,-1 0,-1 -3,-2 0,-2 0,-1 0,-1 1,0 3,-2 4,1 2,2 1,3 1,1 0,1 0,1 1,-1 0,1 1,1 2,0 3,1 -1,-2 3,-1 2,1 0,-1 -1,0 -1,-1 2,-1 1,-2 0,-4 1,-1 -1,0 1,-1 0,-1 0,-1 0,-1 -1,-1 1,-1 -1,0 0,1 0,1 0,1 -1,1 0,1 0,1 0,2 -1,3 -1,2 -5,0 0,-1 1,-3 0,-1 0,1 0,1 -2,2 -1,-1 0,-1 -1,-1 1,0 0,-1 1,-1 -2,1 0,-1 0,-1 2,0 0,1 1,-2 1,0 -1,0 -4,2 0,-1 -1,-1 2,-3 2,1 1,-1 0,-1 1,0 -1,0 -1,0 -1,1 -1,-1 0,-1 2,-1 2,0 -2,-1 -2,1 0,-1 0,-1 1,-1 -1,-1 0,-2 0,-1 0,-2 0,1 -1,3 1,1 0,1 -1,1 1,0 -2,5 -3,1 0,-2 1,-2 0,-1 0,-1 0,-1 -1,2 0,1 -1,1 -1,0 0,2 -2,1 -2,-1 1,-1 1,-1 -2,0 0,-1 -1,-3 2,-3 1,0 0,-1 -1,0 0,-1 0,-2 1,-2 4,-1 2,-2 2,1 4,-1 3,2 1,1 2,1 1,1 0,-1 -2,-2 2,-1 -3,1 -2,-1 0,-1 1,0 2,-1 2,-1 1,1 0,-1 0,-1 0,-1 0,-2 -1,1 -1,2 -1,1 -2,0 -6,0 -2,-1 0,-1 1,-2 1,1 2,1 1,0 1,0 -1,0 -2,-2 0,-1 0,-1 -1,1 -1,0 0,-1 -1,-1 0,-2 -2,-2 -1,-1 0,-1 0,-1 -2,-1 -2,0 -2,-1 -1,-1 1,0 -1,-1 0,-1 0,-1 3,0 1,1 1,-1 -5,-1 -1,-1 0,-2 0,-1 0,-2 0,-1 2,-1 2,2 6,-1 8,7 1,2 0,1 2,0 0,1 0,1 -1,0 1,1 1,-2 3,1 0,-1 3,-1 0,-2 -3,2 -1,1 -2,-1 -1,0 0,-1 -1,0 0,-1 0,-2 0,-1 -3,-1 -5,-6 4,-3 6,-2 0,-1 -1,0 1,-1 7,-3 -1,0 -6,1 4,-5 3,-2 4,-1 0,-1 -1,0 -2,1 -1,-1 -4,3 -1,-1 0,-1 0,-1 0,-3 1,-1 -1,0 1,-1 1,-3 2,-3 0,-1 -4,5 -1,2 0,2 1,0 -1,0 0,1 -2,-2 1,2 0,1 0,1 1,1 0,1 -1,0 1,1 -1,1 -3,5 -5,3 -3,1 0,-1 0,-1 3,-2 1,-3 -1,0 -2,2 -2,2 -1,0 0,-1 1,-2 -1,0 -1,2 0,1 1,1 0,3 -5,1 -3,0 1,-2 -1,2 -3,-1 0,-3 2,-5 3,-3 0,1 7,-4 1,-1 -11,4 -2,3 -2,5 0,2 -1,1 -3,-2 -4,-3 -1,-1 3,-2 4,0 3,-1 2,-2 1,0 -1,0 0,-1 4,-4 -2,0 -3,2 -1,2 -3,2 -3,0 -4,2 -2,0 0,-1 -1,-2 1,-2 2,1 2,-1 -2,0 -2,-2 3,-3 0,-1 1,0 2,-1 3,1 -4,-2 -2,1 -3,3 -2,1 -1,-1 4,-5 -2,1 0,-1 -1,-1 0,1 -1,0 -1,1 0,1 0,1 0,-1 -1,1 -1,-1 -1,-1 0,-1 0,-1 1,-1 2,0 -1,-1 0,-1 3,-1 1,-1 1,1 1,1 0,-2 -1,0 0,-1 2,1 0,-1 -1,-1 3,-3 2,1 1,1 0,1 2,1 -2,-1 1,-1 0,-1 2,0 3,3 -1,-2 0,-1 1,0 1,2 1,1 -1,-3 1,1 0,1 1,0 -1,0 0,-2 -1,0 -5,-2 -1,-2 3,-1 1,-1 -1,-1 0,-1 0,1 0,-2 -1,0 2,-2 1,0 0,1 1,1 1,1 1,-1 1,1 1,1 0,1 2,2 0,1 2,0 2,0 -1,-1 -2,0 -1,-1 -1,-1 0,-1 1,-1 3,3 4,2 1,0 1,0 3,3 1,0 -2,-2 0,-1 -1,0 0,-1 -3,0 -1,-1 0,-1 -2,0 -4,-4 -1,0 0,-3 3,0 -1,-1 -2,-1 0,-1 1,-1 2,1 2,2 1,-1 -3,-3 -1,-1 0,-1 0,-1 3,1 1,1 0,1 1,0 0,-1 2,0 2,0 -3,-1 -1,0 -1,-1 1,-1 -2,0 -1,-1 1,-1 2,-1 1,0 0,1 0,1 1,-2 1,0 0,2 0,-1 1,1 0,-1 0,-1 3,0 1,2 0,1 2,1 2,1 0,3 0,1 1,-4 0,-1 -3,-1 0,-1 0,-1 -1,0 -1,-2 -3,-1 0,-1 1,0 1,0 1,-2 3,1 0,-1 2,0 1,1 2,2 4,2 0,-1 -2,-1 -1,0 0,-1 -1,-1 0,-2 1,-1 -1,0 1,-2 3,-1 0,2 0,1 1,1 2,0 -2,-2 0,-2 1,0 0,-1 3,-1 0,1 1,0 0,2 1,-2 0,1 2,-1 1,0 0,1 1,-1 m 192,293 -1,2 -1,0 -2,-1 -2,1 -1,1 1,0 0,-1 2,0 3,2 -2,3 3,-2 1,0 0,1 -2,1 0,2 -2,-1 -1,1 -1,-1 -3,0 0,1 1,0 1,0 2,1 -2,3 -1,0 -1,-1 -1,0 1,0 1,1 1,0 1,-1 2,0 2,0 -1,0 1,1 1,-1 -1,1 1,0 1,0 -2,1 0,2 -2,-1 -2,1 -2,-1 -1,1 1,-1 1,2 1,0 2,-1 1,0 1,1 -1,0 1,0 0,1 2,0 -3,2 -1,0 -2,-1 1,1 3,1 -2,1 1,0 2,0 0,2 0,-1 -1,1 1,0 -1,0 -1,0 -1,-2 -1,0 1,1 1,1 0,1 -1,0 -1,0 -4,-3 -1,-1 0,1 3,2 0,1 -3,2 1,0 2,-1 1,1 0,1 -2,0 1,0 -1,1 1,0 1,0 2,0 -3,1 3,0 -1,1 2,1 -1,0 1,1 1,0 0,1 -1,1 1,0 0,1 1,0 -1,1 1,0 0,1 0,1 0,1 -1,0 0,-1 0,-1 -1,2 -1,-1 1,-1 -2,1 0,-1 0,-2 -1,0 0,2 -1,0 1,1 -1,1 -1,-1 0,-1 0,-1 -1,0 0,2 -1,-1 -1,-1 0,-1 0,1 0,-1 0,-2 0,-1 0,2 -1,1 1,2 1,2 -1,0 -1,0 0,-1 0,1 0,-1 -1,0 0,-1 1,0 -1,0 0,1 -2,-1 1,-2 -1,1 1,-3 1,0 0,-1 0,1 -2,0 0,1 -1,1 0,1 -1,0 -1,0 0,-1 -1,0 2,-2 0,-1 0,-1 0,-1 -1,0 0,3 -2,1 0,-1 0,1 -1,0 0,-1 1,-1 0,-1 1,-1 1,-1 -2,1 0,2 -2,0 -1,0 0,-2 0,1 -1,-1 0,-2 2,-3 2,-1 -1,0 0,-2 -1,-2 1,-4 -2,3 0,2 1,1 -2,3 -1,0 1,1 -1,1 -3,2 -2,-1 0,-2 2,-2 0,-3 -1,0 0,-1 0,-1 0,2 0,1 0,2 -1,2 -2,1 -1,-2 0,-1 -1,1 -1,0 1,1 -1,0 -1,-1 0,-1 -1,0 0,-2 0,2 -2,0 1,0 -1,-2 0,1 -1,0 0,-1 -1,0 1,-2 -1,1 -2,-1 0,1 2,0 -1,2 -1,0 1,0 -1,1 0,-1 -1,0 -1,-1 0,1 -1,-1 0,1 -1,0 0,1 -1,0 4,-1 1,1 1,0 1,1 -1,1 0,-1 -2,0 2,1 1,1 -1,0 0,1 -1,0 -1,-1 -2,-1 -1,0 -1,-1 0,1 -1,0 2,1 1,1 1,1 0,1 5,1 1,1 0,1 1,1 -1,1 -1,0 0,1 -1,0 -1,-1 1,2 -2,0 -2,-1 0,-2 -3,0 -1,-2 0,1 -1,-1 -1,0 1,1 -2,2 2,-1 3,1 0,-1 2,0 -1,1 1,1 -1,0 1,1 -3,-1 2,1 -1,0 1,1 1,0 0,-1 0,1 0,1 1,0 0,1 1,0 1,-2 1,1 -1,1 0,1 1,0 0,1 0,1 0,1 0,-1 1,2 1,-4 1,-1 1,-1 1,1 0,1 1,1 -1,0 1,0 0,1 -1,0 1,0 2,-1 0,1 1,1 -2,1 0,1 1,-1 1,0 0,1 1,0 -1,1 1,0 1,-1 1,2 -1,1 0,1 1,-1 1,0 0,-2 0,-1 1,1 -1,1 1,0 0,1 -1,0 1,0 0,-1 0,2 1,0 -1,1 2,0 0,1 1,2 0,-1 -1,-3 -1,-1 1,0 1,1 0,-1 0,1 1,0 0,-1 1,1 1,0 1,2 -1,0 1,0 1,1 -1,-2 1,-1 1,2 0,-1 1,1 1,1 -1,-2 -1,-1 0,-1 -1,0 1,0 0,-1 1,1 0,1 0,-1 1,0 0,1 1,1 -1,-1 1,0 1,2 -1,1 1,0 0,2 2,-1 0,1 1,-2 0,2 0,-1 1,0 0,-1 0,-1 0,1 1,-1 0,1 1,1 -1,1 1,0 0,1 -4,2 2,-1 0,1 0,1 1,-1 1,0 0,2 1,-1 1,1 0,1 1,-1 -1,1 3,0 1,-1 1,-1 1,1 -2,2 1,1 -1,1 1,1 2,0 0,-1 3,1 0,1 -1,0 1,1 1,1 0,1 1,0 0,1 -1,0 1,0 1,1 0,1 -1,0 1,0 -1,1 -1,-1 -1,-1 -2,-2 -1,0 -1,-2 -1,0 -1,-1 -1,0 -1,0 4,2 0,1 0,1 1,1 0,1 1,0 2,2 0,1 -2,-1 1,1 -1,0 1,0 1,0 -1,1 1,0 1,0 1,0 1,1 -1,0 1,0 0,1 1,1 -1,0 2,0 1,0 -2,2 -2,0 -1,-1 0,1 -1,0 -1,-1 1,1 1,0 0,1 3,2 2,2 -1,0 -1,2 -2,-2 0,1 0,-1 -1,-1 1,-1 -1,1 -1,-1 1,0 0,-1 -1,1 0,-2 0,2 -1,0 -1,1 -1,-1 -1,0 0,-2 -1,-1 1,2 -1,1 1,1 -1,-1 -1,0 0,-2 0,2 -1,0 0,-1 -1,0 -1,-1 -1,0 1,1 0,1 2,0 0,1 -2,0 -1,0 -2,-1 -1,-1 0,-1 -2,-1 0,-1 0,-1 0,2 2,1 0,1 1,1 -1,0 2,2 -2,0 -1,-1 -1,-2 -1,-1 1,2 -1,0 -1,-1 0,-1 -1,-1 0,-1 -1,0 1,1 0,1 0,1 -1,-1 0,1 -2,-1 -1,-2 0,1 -1,0 -1,-1 -1,0 -1,-1 0,-1 -1,1 0,-1 -1,0 0,-1 -1,0 2,2 0,1 1,1 2,1 0,1 -1,0 -1,-1 -1,0 -2,-1 0,-1 0,1 -1,0 0,-1 -1,-1 -1,1 1,0 0,1 -3,-2 0,1 -2,0 1,0 -3,1 3,1 0,1 3,2 1,0 3,2 1,1 1,1 2,-1 0,2 1,0 0,-1 1,0 1,-1 0,1 -1,1 1,1 0,-1 1,-1 1,0 -1,1 0,1 1,-2 1,1 2,0 -1,1 1,-1 1,1 0,1 1,-1 0,1 1,0 0,1 1,0 0,1 1,-2 1,0 0,1 1,0 1,0 2,1 0,2 1,-1 0,1 1,0 -1,1 2,1 -1,-1 1,0 0,-1 1,1 1,1 -1,0 2,0 0,-1 1,1 0,1 0,2 -1,-1 1,1 0,1 1,0 0,1 1,-1 1,1 -4,3 -2,0 -1,-1 -1,0 -2,0 -1,0 0,1 -2,-1 -1,1 0,-1 -1,0 -3,1 -1,0 0,-1 0,1 -7,1 -2,0 0,-1 -1,1 -1,-1 -1,1 -3,-2 -1,-2 0,2 -1,-1 -1,0 1,1 -2,-2 1,0 0,-1 -1,0 1,-1 -2,1 0,-1 -1,1 -1,-1 1,2 -2,0 0,-1 -1,1 -2,1 -3,-1 -3,0 1,0 -1,0 -2,0 1,-1 -1,0 0,-1 0,1 -1,1 0,-1 1,0 0,-2 1,-1 -1,1 -1,-1 1,0 -1,0 0,1 0,1 -1,0 0,1 0,1 0,-1 -1,0 -1,0 0,-1 -1,1 -2,-1 -1,-1 1,-1 -1,1 0,-1 -1,0 0,-1 -1,-1 0,1 -1,0 -1,-1 1,1 -2,-1 5,-1 0,-1 1,-2 0,-1 1,-1 -2,2 0,-1 0,-1 0,1 0,1 -1,0 0,-1 -1,0 0,-2 -1,1 1,1 -1,-1 0,-1 -1,2 0,-1 -2,-1 0,1 1,0 0,2 0,-1 -1,1 0,-1 -1,0 -1,-1 1,2 -1,1 0,-1 -1,-1 1,0 -1,-1 0,-1 1,-1 -1,0 0,-1 0,1 -1,0 0,2 1,1 -2,0 1,-1 -2,0 0,-1 -1,0 1,-1 -1,-1 -1,0 0,-1 0,1 1,1 -1,0 -2,-1 0,-1 0,-1 -2,0 0,-1 -1,0 0,-1 -1,1 0,-1 -1,1 1,0 -1,0 -1,-1 0,-1 1,0 -1,0 0,1 -1,-1 1,-1 -1,0 1,-2 0,-1 0,-1 -1,1 0,-1 0,1 1,1 -1,1 -1,-1 1,0 -1,0 0,-1 -1,1 -1,-1 0,1 1,1 -1,1 -2,-1 2,2 -1,0 0,1 0,-1 0,1 0,1 0,1 -1,0 0,-1 -1,-1 1,0 -1,-1 1,-1 -2,0 2,-1 -1,0 -1,-2 -1,0 1,1 -1,0 1,1 0,1 -1,-1 -1,-1 0,1 1,1 -1,1 0,-1 -1,0 -1,-1 -1,-1 0,-2 1,0 -1,-1 -1,0 0,1 0,1 0,1 -1,-1 0,1 -1,0 6,3 -1,1 1,0 -1,1 1,0 -1,2 -2,0 -3,-1 0,1 1,0 -1,1 -1,-1 -2,-1 -3,0 1,0 -1,1 1,0 2,1 -1,0 0,1 -2,0 1,0 2,1 -2,1 -1,-1 0,1 -1,0 0,1 -2,0 1,1 1,0 -1,1 -1,-1 1,1 -1,0 -1,0 -1,0 0,1 1,1 -2,0 -3,-1 -2,2 0,-1 -1,0 1,0 0,-1 -2,1 1,-1 -2,0 1,1 -1,0 -1,-1 -1,1 0,-1 -1,0 0,-1 -1,0 0,-1 -1,-1 -1,-1 0,-1 -1,0 1,-1 0,-2 0,-1 -1,-1 1,-2 2,-3 1,0 1,-1 0,-1 -2,-2 0,-1 2,0 -2,-2 1,0 2,0 1,0 2,-1 9,2 1,1 2,0 1,1 0,2 0,1 -2,0 1,0 1,1 1,0 0,2 1,1 1,-1 -1,-2 -1,-1 0,1 -1,0 0,-1 1,-2 1,1 -1,-1 1,0 -1,-2 0,1 0,1 -3,-3 -1,0 -1,-1 -1,0 -1,-1 2,0 0,-1 1,1 3,0 1,-1 0,-1 -1,-1 1,0 2,-1 0,-1 1,0 -1,0 1,1 1,0 1,-1 0,-2 1,-1 0,-1 1,1 1,-1 1,0 1,0 0,1 2,-1 -2,-2 0,-1 -1,-1 0,-1 -1,0 -4,-1 -4,-4 -1,-1 1,-1 2,-4 3,-5 0,-2 1,-2 0,-5 0,-2 1,-1 2,-2 0,-1 0,-2 -2,-1 0,-1 -1,-2 -1,0 0,-1 -1,-1 -1,0 -1,-1 1,-2 -1,0 -2,-1 -1,-1 0,-2 -2,-2 -1,-2 0,1 -1,0 0,-1 -1,0 -1,0 1,1 -2,0 1,-2 1,-1 -1,0 0,1 -1,1 0,2 -1,-1 1,-1 -2,0 -2,-1 0,-2 1,-1 -1,0 0,-1 0,-1 0,-1 -4,1 4,2 0,1 -1,1 0,-1 -1,1 -2,0 -1,-1 -1,-1 -1,-1 -1,-1 1,-1 -1,0 1,0 -1,1 -2,0 2,-2 0,-1 -3,1 1,-1 -1,1 1,-1 -1,1 0,-2 2,-1 -1,-1 -2,2 1,2 0,1 -4,-2 0,1 1,1 -1,1 -4,6 -2,0 -1,-1 1,-3 -1,0 0,-1 -1,-1 2,-2 3,0 1,-2 1,0 0,-2 -1,-1 -3,-2 -2,0 -1,1 -1,-1 -2,-2 0,-1 1,-1 2,0 -1,-1 -1,0 -2,0 1,1 0,1 0,1 -2,0 0,-1 0,1 -2,-1 2,-1 0,-2 -1,1 0,-1 1,-1 -1,0 -1,1 0,-1 -1,1 -1,0 0,-1 1,-2 -1,1 -2,3 0,-1 1,1 0,1 -1,0 -1,-1 -2,-7 -1,-2 -1,2 -2,-1 1,1 -1,0 0,1 -1,-1 0,-1 -1,0 1,-1 -1,0 -1,0 0,1 0,-1 -1,0 0,-2 -1,1 0,-1 0,1 -2,-1 0,-1 1,0 -1,0 1,-1 1,0 -1,-1 -1,0 0,-1 -1,2 1,-1 -1,3 1,0 0,2 -1,0 0,-1 0,-1 -1,1 0,1 -1,1 1,1 1,-1 1,1 1,0 0,-1 1,1 2,2 1,2 1,1 0,1 -1,2 -4,1 -2,1 -2,-2 0,1 -1,-1 -1,0 -1,0 -2,-1 -3,1 0,-1 -3,1 -2,0 0,1 2,0 1,1 0,-1 1,1 1,1 3,2 0,1 -1,1 0,-1 -1,0 -1,-1 -3,-2 -2,0 -4,-3 -2,0 -3,-2 -1,0 2,2 6,2 1,1 2,1 0,1 -1,0 0,1 -1,-1 -1,1 1,1 -2,0 1,0 -1,0 1,-1 -1,0 -1,1 -5,-3 -1,0 -1,1 -2,1 -7,0 -1,0 -1,-1 1,-1 -1,1 -5,0 -1,1 0,1 1,-1 3,1 -3,1 -4,-2 -3,-2 0,-2 0,-1 0,-1 1,-2 0,1 -2,2 0,1 -1,0 0,-1 1,0 -2,0 1,-1 -1,0 0,-1 -1,1 0,1 -1,0 2,0 0,1 0,1 -5,0 1,1 -2,1 -1,-1 -2,-1 1,0 1,1 1,-1 -4,-2 -2,0 -2,-1 -1,-2 0,-1 -2,-3 -1,-2 1,-1 -2,-2 2,0 1,1 3,-1 2,0 0,1 2,1 5,-1 0,-1 0,-1 2,0 -3,0 -2,-2 -1,-1 0,-1 -1,0 -1,1 -10,-1 -2,-1 -1,-2 0,-1 0,-2 1,-3 -1,-1 0,-1 -1,0 -1,-1 1,-3 1,-2 1,0 0,-1 -1,1 -1,-1 0,-1 0,-1 0,-1 0,-1 0,-1 1,0 -1,0 0,-1 0,-3 1,0 1,0 -1,-1 1,0 -1,-1 0,-1 0,-1 2,1 0,-1 -1,-1 1,-2 -1,-1 1,-1 -1,0 1,-2 0,-1 0,-1 1,0 -1,-1 1,-1 2,0 -2,-1 4,-6 4,-3 5,-3 7,-1 3,2 -1,1 -3,4 -1,4 -1,2 -1,5 -1,3 0,3 0,1 0,1 2,2 2,2 -1,4 0,1 0,1 1,3 1,1 2,3 2,3 2,2 4,2 1,0 0,1 1,1 -3,1 -2,1 1,0 0,1 0,-1 -1,1 -1,0 -1,1 -2,2 -4,2 2,1 2,-1 3,-4 2,-1 2,1 0,-1 2,0 0,2 -1,-1 -1,1 1,0 0,1 1,0 0,1 0,1 1,0 0,-2 -1,-1 0,-3 1,-1 -1,-1 0,-2 1,-2 -1,-1 0,-1 0,-1 -2,-1 0,1 -3,0 -2,-2 1,-1 -1,-2 -4,-1 1,0 0,-1 1,0 1,-1 0,-1 1,-1 -1,0 0,-1 1,0 -1,-1 4,0 4,4 1,1 0,-1 0,-1 -1,0 -1,-1 -1,-1 -1,-1 -2,-1 1,-2 2,0 0,-1 -2,1 -1,-1 1,0 0,-1 -2,3 0,-1 -2,0 -1,-1 -1,-2 1,-1 -1,0 -1,-1 0,-1 0,-1 0,-2 0,-1 0,-2 2,0 9,3 -8,-3 0,-1 -1,0 -2,0 1,-2 1,1 0,-1 0,-1 2,1 6,0 2,1 -1,-1 -5,-1 -4,-1 0,-2 0,-1 1,-1 1,2 3,1 -1,-1 -1,-1 0,-1 -1,0 3,-2 2,0 1,2 1,1 0,-1 -1,-1 0,-1 -2,-1 4,-4 1,0 0,-2 8,-2 1,1 0,1 1,1 1,3 1,3 4,2 1,1 0,1 0,1 0,1 1,2 2,1 0,1 -1,2 0,1 -2,3 0,3 0,1 -1,1 1,0 1,-4 2,-2 0,1 0,1 -1,3 0,1 -1,1 4,2 -1,0 -1,-1 -1,1 -1,1 2,-1 0,1 1,1 -1,2 1,-1 0,-1 2,-2 0,-1 -2,-2 2,0 -1,-1 0,-1 1,0 2,2 2,0 -3,-1 -1,-2 0,-1 1,-2 0,-1 1,0 0,2 1,1 0,1 0,1 0,1 1,-2 1,1 -1,-1 1,-3 2,2 1,0 0,1 0,1 0,1 0,1 -1,3 2,-1 3,2 0,-1 -1,-1 -2,-1 0,-1 0,-2 2,1 2,3 2,2 2,0 -2,0 -2,-2 1,-1 2,2 1,1 1,1 0,-2 -2,0 -1,-2 -1,0 -1,0 -1,-2 0,-1 -4,-1 1,-1 -1,-1 1,-1 1,0 0,2 1,1 1,0 -1,-1 0,-1 -1,-1 1,0 5,1 0,1 0,1 1,0 -1,-1 2,-1 2,1 -2,-1 -1,0 -7,-2 -1,-1 0,-1 0,-2 4,-5 3,-1 2,0 1,1 0,1 4,0 0,1 1,1 -1,-2 1,0 5,0 2,3 1,1 -1,3 -2,1 -2,2 -1,5 0,-2 1,-1 0,-2 1,-1 1,-1 2,0 1,0 -1,1 -1,2 0,3 0,2 -1,1 1,0 0,-1 0,-2 0,-2 0,-1 2,-2 0,-2 6,-1 1,2 0,1 0,1 0,1 -1,1 -1,0 1,0 -1,2 -4,2 2,-1 1,1 -2,4 3,-5 1,1 0,1 -2,1 0,2 0,-1 2,-1 0,1 -1,1 2,0 0,1 -1,4 0,1 1,-2 1,-3 -1,-1 -1,-1 0,-1 1,-1 0,-2 0,-1 1,-1 1,0 0,1 0,2 1,1 -1,-2 0,-1 0,-2 2,-1 0,1 0,-1 1,1 -1,1 0,1 0,1 0,1 0,1 0,1 -1,2 1,-2 0,1 1,1 -1,1 1,0 -1,3 1,-3 0,-1 0,-1 0,-1 0,-2 0,-1 1,0 0,1 0,1 1,1 0,1 1,-1 0,-1 1,1 -1,0 0,3 0,3 0,2 1,-2 -1,-1 0,-1 1,-2 0,-1 2,0 -1,2 1,1 0,1 0,2 0,1 0,-1 0,-1 0,-2 0,-1 1,-1 0,-2 -1,-1 1,-1 -1,-2 1,-1 0,-2 7,0 1,1 0,1 1,0 2,2 0,2 0,3 -3,0 -1,1 0,1 0,1 0,1 0,1 -3,3 3,-2 1,0 0,2 0,1 -1,2 1,-2 1,-2 2,-3 2,-1 1,0 -1,-1 0,-2 1,-3 2,1 2,2 -1,2 1,2 -2,3 0,1 -2,1 0,1 0,2 -1,1 1,0 1,2 -1,1 1,-1 0,-1 -1,-2 1,-2 -1,-1 3,0 1,4 -1,1 1,1 0,5 1,-3 0,-1 0,-1 -1,-1 0,-5 -1,0 0,-2 1,-1 1,-2 3,-5 1,0 0,2 0,3 -1,1 -1,1 1,1 0,2 1,-1 -2,-2 2,-2 1,1 -1,-1 2,-1 -1,-1 1,0 0,-1 1,-1 1,-1 1,1 7,1 0,1 0,2 0,1 -1,0 0,-1 0,2 -4,4 -1,2 -1,1 0,1 -1,2 2,-1 1,1 -2,1 -1,2 0,1 1,-3 2,-1 -1,-1 0,-1 1,-1 2,-2 1,0 0,-1 -1,0 3,-1 1,1 0,1 0,1 -1,2 -1,4 -2,1 -1,0 0,4 1,1 0,-5 2,0 1,-4 1,-1 2,0 -1,-1 1,-1 -1,-1 2,-1 0,-1 -1,0 0,-1 0,1 0,-1 0,-2 0,-1 2,0 1,0 2,1 4,3 3,4 -2,1 -3,0 -2,1 0,2 0,2 -3,0 -3,4 -3,1 -2,1 1,0 1,-1 3,0 4,-4 5,1 1,-1 4,-2 4,1 1,1 1,1 0,2 -2,0 -1,1 -1,0 -2,1 -5,-1 -1,1 -5,1 -1,2 2,-2 4,0 2,-1 1,1 -1,0 -5,2 3,-1 -1,2 -2,2 1,-1 1,2 -1,1 1,-1 -1,-1 2,-3 3,-1 4,1 -2,2 -2,0 -1,2 1,-1 2,0 0,1 1,0 -3,1 5,0 -6,1 1,1 -6,1 2,0 -2,1 3,-1 1,1 -1,0 1,0 3,-1 3,0 1,1 -2,0 0,1 -3,0 -1,2 1,-1 1,-1 1,1 -1,1 2,-1 0,-1 2,0 0,1 1,-1 1,1 0,2 1,-1 -1,-2 1,-1 1,0 0,1 1,2 0,-1 -1,-1 2,-1 1,0 0,1 1,0 0,-2 1,1 1,-1 1,0 -1,0 0,1 -1,0 1,1 -1,0 -3,1 -2,2 0,1 0,1 0,-2 2,-1 0,1 0,1 0,1 0,-1 0,-1 1,-1 1,-1 2,0 -2,1 0,2 1,1 0,1 -2,0 2,0 0,-1 -1,-2 2,-1 1,0 1,0 0,1 1,1 -1,2 1,1 0,-2 0,-2 1,1 0,1 0,1 1,0 -1,-3 2,-1 0,1 0,1 1,-1 1,1 -1,1 1,2 -3,1 3,0 1,1 -1,-2 0,-2 1,-2 -1,-2 1,-1 0,1 0,1 1,0 0,3 1,1 0,1 2,4 -1,-3 -1,-1 1,-2 -1,-1 2,-2 1,1 0,3 2,1 2,2 -2,-2 -2,-1 1,-1 -1,-2 1,-1 0,-1 0,-1 1,-1 2,1 0,1 -2,1 2,0 -1,1 0,2 1,-1 0,2 0,1 1,0 -1,-1 1,-2 1,0 0,1 1,0 -1,-2 1,-1 1,0 0,1 1,-1 0,1 -1,1 0,1 1,-1 1,-1 1,0 0,1 0,1 1,-1 1,1 0,2 -2,1 3,-1 2,1 0,1 -6,2 4,-1 0,2 -2,0 -1,2 1,-1 4,-3 1,1 -1,1 -3,3 0,1 1,1 0,-2 0,-1 3,-3 0,-1 1,0 2,-1 1,1 0,1 -1,2 0,1 1,1 -1,-1 1,-2 1,-2 -1,-1 0,-2 0,-1 0,2 1,0 0,-1 -1,-1 1,-1 1,1 1,1 0,1 -1,2 1,2 0,1 -1,2 0,1 -1,1 2,-1 -1,-1 1,-1 1,2 0,2 1,0 -1,-2 -1,-2 3,-3 2,3 1,3 0,-1 0,-1 0,-1 -1,-1 -1,-2 1,-1 1,1 1,1 1,0 1,1 0,-1 -2,-1 0,-1 0,-1 -1,-1 0,-1 2,-1 1,3 1,0 -1,-2 1,0 0,1 1,-1 0,1 -1,1 3,-1 0,1 -1,0 3,0 2,1 1,1 m -246,9 -1,0 1,1 3,-1 -1,0 1,1 2,0 -2,0 1,2 3,1 0,-1 1,-1 0,-1 1,-1 3,0 -1,1 -1,1 -1,1 1,0 1,-1 1,1 1,-1 0,1 1,0 1,-1 1,1 2,1 -2,0 -1,0 1,1 -2,1 0,2 -1,0 1,0 -1,0 0,-1 -1,1 1,-2 -2,2 -1,-2 6,6 -1,1 1,1 0,1 2,1 -1,0 1,1 -1,0 1,0 0,1 -1,0 1,1 0,1 -1,1 1,0 -1,1 1,1 -1,2 2,1 -1,1 1,-1 1,2 1,1 0,-1 0,-1 1,-1 0,-2 1,-2 -1,0 0,-1 0,-1 1,-1 0,1 0,-1 0,-3 0,-2 0,-2 1,-2 1,-2 2,-1 1,0 1,2 0,1 5,3 2,3 1,2 0,1 1,4 0,1 1,1 -1,1 0,2 -1,-1 -1,0 -1,-3 -1,2 1,4 0,1 0,1 0,1 1,3 1,1 1,2 1,1 4,4 1,2 1,0 -1,2 0,1 0,-1 1,0 1,1 0,-2 0,-1 1,1 0,-1 1,0 -1,-1 0,-1 1,0 0,-1 1,-1 1,1 0,-1 0,1 1,0 0,-1 0,-1 0,-1 0,-2 -1,-1 1,-3 1,-3 0,-2 1,1 1,-2 -1,-1 1,0 -1,-1 1,-1 0,-2 -1,0 1,-2 0,-1 0,-1 -1,-2 0,-3 2,0 0,-1 -1,0 3,1 -1,-1 4,0 0,-1 -1,-1 -1,1 -2,-1 2,0 -1,-1 1,0 -1,0 -1,-1 1,0 2,0 -2,-1 1,0 0,-1 2,1 1,-1 -2,0 0,-1 -2,0 0,-1 -1,1 0,-1 1,0 -2,0 0,1 -1,-1 1,0 -1,-1 -1,-2 1,0 -2,-1 1,-1 0,-1 -1,1 0,-1 0,-2 1,0 -2,-1 0,-1 1,-1 -1,-1 1,-1 0,1 0,-1 -2,0 0,-1 2,0 2,1 1,-1 1,0 2,-1 6,2 3,0 0,-1 2,1 2,-1 3,0 0,1 -2,0 1,1 -2,0 1,0 1,2 -5,-1 0,1 3,0 5,1 -1,0 2,2 -1,0 0,1 0,1 2,-1 0,-1 2,0 2,1 1,0 1,2 -1,1 -1,1 -1,1 -2,2 3,-1 1,1 0,1 0,-1 1,0 2,1 -1,1 1,2 -2,3 -2,0 1,0 0,1 -1,1 -1,0 0,-1 -2,0 1,0 0,1 -3,0 1,1 -1,0 0,1 1,0 1,0 0,1 -1,0 1,1 1,0 0,1 0,1 0,1 2,0 0,-2 1,0 0,2 -1,0 0,2 1,2 2,1 1,1 3,3 2,2 1,1 -1,1 0,2 1,3 0,2 -1,0 1,1 -1,0 -1,1 -1,0 -1,1 0,1 0,1 0,1 -1,1 1,1 -2,0 0,1 -1,1 0,2 -3,1 0,1 -3,3 -1,1 0,-1 -1,0 0,-1 1,1 -2,-3 -1,0 -1,-1 0,1 -1,-3 1,0 0,1 0,-1 -1,-1 1,-1 -1,-2 0,2 -1,1 0,1 -1,0 -1,0 1,-1 -2,-1 2,0 -2,-1 0,-1 -2,-1 2,0 -5,-1 1,1 0,1 1,-1 1,0 -3,1 -1,2 1,1 1,-1 0,-1 -1,0 2,-1 1,0 0,1 1,0 -2,0 3,1 -2,0 3,1 0,2 1,0 1,0 0,1 0,1 3,2 0,1 1,2 2,2 -2,1 1,-1 -2,0 -2,-2 -1,1 -1,0 0,-1 -1,-1 -1,2 3,2 0,1 -1,0 -1,-1 -1,0 -1,-1 -1,1 -1,-1 -1,-1 0,2 -1,-1 0,-1 -1,-1 0,-2 -1,-1 -3,2 -2,0 -2,-1 -1,1 -1,-1 0,1 0,1 -2,-1 2,2 -2,0 2,2 3,1 4,2 -1,1 0,1 -1,2 0,1 -1,0 -1,1 0,2 0,1 0,1 -1,1 -1,2 -2,1 1,0 -1,1 0,1 0,1 -2,1 -5,1 -3,-1 -1,-2 -2,-2 -2,-1 3,0 -4,-1 0,1 -2,-1 0,-1 -3,-2 0,1 -1,-1 -1,-1 -3,2 -3,0 0,-1 2,0 -1,0 -3,1 -6,-1 -1,0 4,3 -1,-2 0,-1 5,1 5,1 5,4 0,2 2,1 2,2 8,0 3,-1 3,0 3,1 1,2 0,1 -1,2 0,1 -1,1 0,2 0,1 -1,1 -1,-1 0,1 0,1 0,2 -2,1 0,4 -1,2 0,1 -1,1 -1,0 -4,4 -1,0 0,-1 -1,1 -2,-1 -1,-2 1,2 1,2 1,0 -1,0 -2,-1 1,1 -1,-2 -1,1 -1,-1 0,1 1,1 -2,-2 -1,0 1,1 -1,0 -1,-1 0,-1 1,0 0,-2 -1,1 0,1 -2,0 2,0 -1,1 1,1 0,1 1,0 0,1 -2,0 1,-1 -1,0 -1,1 -1,-1 -1,0 2,1 0,1 1,2 -1,0 1,1 -1,2 0,1 -1,0 1,-1 -1,0 -1,1 -1,0 0,-1 -1,-1 -1,1 1,0 1,0 0,1 -4,1 -1,-1 -3,0 0,-1 1,0 -1,0 -3,0 1,-1 -1,-1 0,1 -2,0 0,-1 -1,1 -1,0 -2,0 -1,-1 -2,-1 -1,0 -6,-3 -1,-1 -1,-1 1,0 -2,-1 1,1 0,1 1,1 0,1 -2,0 0,1 1,0 1,1 2,2 0,-1 1,0 -1,-1 -1,-1 0,-1 1,0 1,1 5,2 1,1 1,-1 1,2 1,-1 2,1 -1,1 -1,1 0,-1 -1,2 -1,0 0,1 2,-1 1,1 0,-2 4,-2 1,1 0,1 2,1 0,1 1,0 2,1 1,-1 2,2 -2,-1 3,1 0,1 -1,0 1,1 0,1 0,1 0,1 1,1 -1,0 1,0 -1,1 0,1 0,1 -3,0 -1,1 0,1 -2,1 1,0 -8,-1 1,1 -3,0 -1,0 2,2 1,1 -1,0 3,0 1,1 -1,2 -1,0 -1,0 1,-1 -1,0 -1,0 1,0 -1,0 0,1 -1,0 -1,0 0,-1 -1,-1 1,2 0,1 0,1 -2,0 1,1 -2,1 3,1 0,1 -1,0 2,2 -1,0 -1,-1 -1,-2 -4,-1 4,2 0,1 -2,0 0,1 -1,0 0,1 1,0 1,0 -1,1 -1,0 0,2 -2,-2 0,1 -1,0 2,1 -2,0 1,1 1,0 -1,1 -3,-2 1,2 -1,0 3,1 -1,0 2,1 -2,0 0,1 1,0 -3,2 -1,2 -1,-1 0,1 -1,2 0,1 1,0 1,1 -1,1 -2,-1 1,2 -1,2 1,0 -1,1 1,0 -1,1 -1,3 -1,1 -1,2 0,1 0,1 -1,2 0,1 -2,-1 1,1 -2,0 2,1 0,2 -1,2 1,1 -1,2 1,1 -1,0 0,1 -49,-1 -5,0 -20,-1 8,-86 -62,-22 -8,-15 -15,-4 -47,-69 12,-29 1,2 2,3 7,5 1,1 1,3 3,4 3,1 1,2 2,0 -1,0 1,0 0,1 1,0 2,1 1,0 0,1 2,1 -1,-1 1,0 -2,-2 1,-1 2,2 1,0 -1,-1 3,1 3,3 1,2 1,1 1,2 0,2 1,1 0,1 1,1 0,1 -1,0 1,1 0,-1 0,1 -1,1 0,2 -2,-1 -1,-1 -2,0 -2,-1 1,2 -1,1 -2,-2 0,1 -1,1 0,1 0,1 -3,1 2,1 -1,1 1,1 3,1 1,1 1,0 1,1 1,0 2,2 5,1 1,1 2,1 3,0 0,-1 1,0 2,1 0,1 1,0 0,-1 3,1 -1,-1 1,0 0,1 2,-1 1,-1 -1,1 1,0 -1,1 1,-1 1,1 1,0 1,-2 4,0 0,-1 1,1 0,-1 1,0 1,1 -1,2 0,-1 1,1 1,1 -1,1 1,1 -1,0 2,1 0,-1 1,0 2,1 1,4 -1,1 0,1 0,1 1,0 0,1 1,-2 0,-3 1,0 1,0 -1,1 1,2 -1,0 0,2 1,0 0,-1 1,0 2,3 0,3 0,1 0,1 -2,-1 -1,1 -2,-3 0,3 1,-1 0,2 1,0 1,1 -1,2 -1,0 2,1 0,1 0,1 1,1 0,-1 0,-1 1,3 0,1 1,1 0,2 2,2 0,-1 -2,-4 0,-2 -1,-2 0,-3 1,-2 1,1 0,-2 1,1 0,3 -2,-1 0,1 1,0 0,1 1,1 -1,-2 1,0 0,-2 2,4 0,-3 -1,-2 1,0 1,-1 -2,-2 -1,-5 0,1 1,-2 -3,-5 1,-1 0,-2 2,0 1,-1 -2,-2 1,-2 1,1 -1,0 1,1 -1,0 0,1 1,-2 3,1 1,-1 1,0 1,1 0,-1 1,0 1,-1 -1,-1 1,0 0,1 1,0 0,-1 1,1 0,1 1,-1 -1,-1 0,-1 0,-1 2,-1 1,1 1,-1 0,1 0,-1 1,0 1,0 1,-1 -1,-1 1,-2 -1,0 -2,0 -4,1 1,1 -1,0 0,1 -1,-1 1,2 0,1 0,1 -1,0 0,-1 -1,0 1,-1 -1,-1 0,1 -2,0 -1,2 -1,0 0,-2 -1,0 -1,0 -2,-1 -1,2 2,2 -1,0 -2,0 -1,-1 -2,1 0,-1 0,1 1,0 0,-2 -1,0 0,1 0,-1 1,-2 -1,0 -2,1 -1,-1 1,0 0,-1 2,-1 2,-4 8,0 4,-2 1,0 4,-2 3,2 2,3 -1,2 1,3 0,1 -1,-1 0,1 0,1 1,0 0,1 2,2 0,-1 1,0 0,1 -1,1 1,0 2,0 0,1 -1,1 1,0 0,1 0,1 0,1 2,-1 1,1 5,0 0,-1 1,-1 0,1 0,2 2,1 0,1 1,0 1,2 2,1 1,2 1,0 0,1 0,-1 1,-1 1,1 0,-1 1,1 1,0 0,1 2,0 0,-1 2,1 0,-1 0,-1 0,1 1,-1 0,1 1,0 0,-1 1,0 1,-1 1,0 0,1 1,-1 5,2 0,-1 1,1 1,0 1,0 1,2 1,-1 2,1 0,-1 1,0 1,-1 0,1 2,-1 -1,-1 -2,-2 0,-3 2,0 2,4 1,3 1,0 0,1 1,1 1,1 1,1 2,-2 1,1 0,-1 1,0 0,-1 -1,0 1,-1 -1,-3 -1,1 -3,-3 0,1 -1,0 -1,1 0,-1 0,1 0,2 -1,0 -3,-7 1,-1 0,1 0,1 1,-1 -2,-1 0,-1 -1,-2 0,-1 1,1 1,0 -1,-1 0,-1 3,3 0,-1 1,1 -1,-1 2,0 0,-1 1,1 0,-1 -1,0 0,-1 -1,-1 0,-1 1,0 2,1 0,1 1,1 0,-1 -1,0 1,-1 2,1 1,1 0,1 1,0 0,1 -1,0 0,1 1,0 2,0 1,0 -1,1 1,1 0,-2 1,-1 0,1 0,1 -1,1 0,1 -2,1 0,1 1,0 1,-1 0,1 1,0 0,-1 -1,0 1,-2 4,-3 -1,3 0,1 -1,6 0,2 0,1 0,1 -1,0 0,-1 0,1 -1,2 0,1 1,1 1,0 1,-1 0,1 -1,2 -1,2 1,0 1,1 1,-2 1,-2 1,1 0,-1 1,0 0,1 -2,4 1,-2 0,1 1,0 1,1 0,-1 0,1 0,1 0,1 1,0 0,2 -1,0 -2,0 1,-1 -1,0 -2,1 0,-1 -1,0 1,-2 -1,2 0,1 -1,0 0,-1 -1,0 0,-1 -1,-1 0,1 0,1 2,2 1,1 2,2 0,2 0,1 1,-1 0,-2 0,1 -1,0 0,-1 -2,-2 0,-2 2,0 1,1 2,-1 1,1 0,-1 1,-1 -1,0 0,-1 0,-1 0,-1 0,-2 0,-1 1,0 -1,-1 -1,-2 1,-2 0,-1 -1,-1 -2,-2 1,-2 0,-1 0,-1 1,-2 -1,-1 2,0 0,1 1,-1 1,0 2,0 1,-2 3,-2 0,-2 1,-1 3,-3 1,0 0,-1 1,-1 -3,0 1,-2 0,-1 0,-1 1,0 -1,-1 0,-2 -2,0 0,-1 0,1 1,-2 -1,1 0,1 -1,2 0,1 1,-1 0,1 1,-1 -2,3 -3,1 -1,-1 0,-3 0,-1 0,-1 1,1 1,-2 1,0 0,-1 0,-1 1,-1 -2,1 0,-2 0,-1 0,-1 2,-1 1,0 1,-1 1,-1 -2,4 0,1 0,1 3,-4 -1,0 1,-2 -1,-1 -2,0 0,2 -3,0 -1,-2 -1,-3 -1,1 0,-1 -1,0 1,1 -1,1 -1,-1 -1,0 -2,-1 -2,-2 -1,0 0,1 0,-1 0,-1 -1,1 -1,-2 -1,0 1,0 0,-1 0,-1 -1,0 -2,-3 -1,-3 1,-2 0,-1 1,0 0,-2 1,-2 0,1 1,-1 2,1 -2,-2 2,-2 -2,2 -1,0 -1,-1 -1,-1 -1,-1 0,-2 1,-1 0,-2 1,-1 0,-1 0,1 -1,0 0,-1 0,-1 1,0 -1,-1 1,-1 -1,0 1,-1 1,-2 1,0 1,0 2,3 1,-1 -1,-1 1,0 -1,-1 2,-2 -1,0 -2,0 -1,-1 0,-1 1,-1 2,-1 1,-1 0,-2 2,0 2,0 0,-1 -1,1 -1,0 -1,0 -1,-1 0,-1 5,-1 1,1 -1,1 0,1 -1,1 1,1 0,-2 1,1 0,-1 0,1 0,3 1,-3 1,1 0,-1 1,1 1,-1 0,1 0,2 4,3 0,2 2,1 -1,0 1,1 0,2 1,1 -1,2 1,0 -1,3 1,1 -1,0 0,1 2,1 0,1 0,1 2,0 1,0 0,1 0,1 0,-1 1,0 -1,1 1,1 -1,0 1,1 1,1 0,2 1,1 0,-1 0,-1 1,0 0,1 1,2 0,-1 1,2 -3,1 -2,-1 -1,-2 0,1 1,0 0,1 -2,0 1,1 -1,1 -1,0 1,1 1,-1 2,0 1,1 -1,1 -1,1 0,1 -2,2 -1,0 0,1 -1,1 2,0 -1,0 m -111,-66 1,1 -1,-2 0,-1 1,1 0,-2 0,-1 0,-1 1,1 0,-1 1,0 -2,-1 1,-5 1,1 0,1 1,0 -1,1 2,2 0,1 1,0 1,0 -1,1 1,1 -1,2 1,0 0,-1 1,0 1,2 0,1 -1,1 0,2 0,3 0,1 1,0 -1,1 0,2 1,1 -1,1 1,-1 -1,2 0,1 0,1 1,1 -1,0 1,0 0,2 1,1 1,0 -1,-2 1,0 1,0 1,-1 2,0 -2,-2 2,0 -1,-1 0,-1 -1,-1 0,-1 0,-1 -1,-1 1,-2 0,-1 0,-1 0,-1 0,-1 1,0 -1,-2 0,-1 0,-1 0,-1 0,-1 0,-1 0,-3 0,-1 0,-1 0,-1 1,0 -1,-2 1,-1 2,0 -1,-1 0,-1 0,-1 3,1 3,4 0,-1 -1,-1 1,-1 1,0 1,0 0,2 1,0 0,1 1,2 1,1 1,0 1,0 0,1 1,2 -1,0 1,1 0,1 1,1 -1,1 1,0 -1,0 0,-1 0,1 0,1 1,1 -1,1 0,1 1,0 0,1 -1,0 1,1 -1,0 1,6 0,1 0,1 0,1 0,1 1,3 0,1 2,5 -1,2 1,0 -1,1 1,2 -2,0 0,2 -2,2 2,3 1,2 1,3 1,1 2,3 1,0 1,2 2,1 0,-1 -2,-1 0,-2 1,0 1,1 0,2 3,3 1,1 1,0 1,2 2,-1 0,1 0,1 0,2 1,0 1,-1 1,1 0,-1 1,0 1,1 -1,1 1,1 0,2 -1,0 1,1 0,2 0,1 -1,0 0,1 0,1 -1,0 0,-1 -2,0 -1,-3 1,-1 0,-1 -2,6 -1,-2 -1,0 -1,-2 1,0 -1,-1 0,1 0,-2 -1,0 0,1 -1,0 1,1 -1,1 0,-1 0,1 -1,-1 -1,1 0,1 1,0 0,3 -3,0 -2,-2 -1,-1 -1,0 0,1 1,2 1,2 0,-2 1,1 0,1 -1,2 0,1 -1,2 0,1 0,1 3,-6 2,0 1,-1 3,0 0,1 0,2 -1,-1 0,1 0,1 0,1 -1,-1 0,1 1,1 0,-1 1,0 1,2 -1,2 1,1 -2,1 0,-1 -1,1 -2,-1 0,1 -1,1 -2,-1 1,2 -2,0 -1,0 -1,-1 -1,0 0,1 -2,-1 -4,-1 -1,-1 0,-1 0,1 -1,-1 -1,1 0,1 -4,-2 -1,-1 1,-1 1,0 1,0 0,-1 -1,-1 0,1 -3,-1 -1,-2 -4,0 -2,-1 1,-1 1,-1 0,-1 0,-1 -2,-3 -1,1 0,2 -2,0 0,3 -1,0 0,1 -1,0 -2,2 -5,1 -4,-1 0,1 -4,3 -3,0 -2,1 -4,-1 -2,1 0,-1 -1,0 -3,0 -1,0 2,0 0,-1 -4,0 1,1 0,1 -3,-2 -4,0 -4,-1 0,1 -4,-1 -1,0 0,-1 2,0 -1,-2 -1,-1 0,-1 -1,-3 1,0 0,-2 1,-1 0,-1 1,0 -1,-1 0,-1 0,-1 2,0 -5,-4 -4,0 -1,-1 -2,1 -2,-1 -3,-2 -3,-4 -1,-1 1,0 0,-1 0,-1 -1,0 -1,-1 1,-1 2,1 -2,-6 1,-2 28,8 0,3 1,1 1,-1 1,0 0,-2 17,4 11,-50 m 25,-81 -1,1 2,0 1,3 -1,0 0,1 -1,1 -1,0 0,1 0,1 -1,1 4,0 1,1 -1,2 -1,0 -1,2 1,0 1,-1 2,-1 1,1 0,1 0,1 0,1 0,1 0,1 0,-1 1,-1 1,-1 -1,-1 0,-1 0,-1 2,-2 3,1 2,2 0,2 0,1 0,1 0,1 0,1 -1,2 -1,2 0,1 0,1 -2,2 0,1 0,1 0,1 -1,0 0,1 -4,1 -1,1 -2,0 -2,-1 0,-1 0,-1 0,-1 0,1 -2,1 0,1 -2,0 -2,-2 1,1 0,-2 -1,-1 -1,1 0,1 -1,0 1,0 -2,0 -1,1 -1,1 -2,-1 -1,1 4,-14 1,0 4,2 0,-1 0,-1 1,0 0,-1 0,-1 -2,-3 -1,0 1,0 -2,-1 1,0 0,-1 -1,0 -1,-1 2,-1 2,-1 0,-1 1,-1 -1,-1 -1,-1 -1,-1 0,1 0,-1 1,-5 1,0 1,-1 -1,0 0,-1 1,0 0,-1 1,0 0,-2 1,0 1,-1 2,0 0,-1 1,1 -1,1 1,0 0,1 0,1 0,1 -1,0 0,1 0,1 0,1 0,1 m 1,-36 1,1 -1,2 -2,0 1,-3 1,0 m 2,-12 2,3 0,1 -1,2 0,1 0,1 -2,0 -1,-1 2,-7",
                        "NT": "m 380,227 1,0 -1,0 m -78,50 0,2 -1,-1 1,-1 m 28,-40 1,1 -1,-1 m -135,83 -1,-3 1,1 0,1 0,1 m 126,34 -1,-1 1,0 2,1 -2,0 m -69,-31 -1,0 -1,0 2,-1 0,1 m 107,93 -1,0 -2,1 3,-2 0,1 m -6,-49 3,2 -3,-1 -1,-1 1,0 m -15,-127 0,2 -2,1 -1,0 2,-1 0,-1 0,-1 1,0 1,0 -1,0 m 38,-5 -3,-2 2,-2 4,1 1,1 0,2 -2,1 -2,-1 m 6,-20 -2,-2 1,0 -2,-5 1,0 1,-1 2,0 0,1 1,1 0,1 1,1 0,1 0,1 -1,1 -2,1 m -26,27 0,1 0,1 -2,2 -1,1 -1,1 -1,1 -2,2 -3,2 -1,-2 -1,-1 -1,-1 6,-5 2,0 1,0 1,-1 1,0 1,-1 1,0 m -168,80 1,1 1,0 -1,0 0,-1 -1,-2 2,-2 1,3 1,2 -1,-2 0,-3 1,-1 1,1 -1,2 1,-2 2,3 0,-1 1,-2 2,-1 0,1 2,0 1,2 1,0 0,-1 -1,1 0,-1 1,-1 0,1 0,-1 2,-1 0,-1 1,0 0,1 0,1 0,1 -1,0 1,0 -1,1 1,0 0,1 0,1 1,-1 1,0 0,1 -1,0 -3,1 -1,1 0,1 -1,-1 -1,1 -1,1 -1,0 -1,0 -2,1 0,1 0,2 -1,2 1,1 -1,1 -1,-1 1,-1 -1,-1 1,-1 0,-3 -1,0 0,1 -1,0 -1,-1 0,-1 -1,-1 0,-1 -1,1 -2,-1 -1,-1 1,0 0,-1 1,-1 0,1 m 181,-96 1,2 -1,1 -2,-2 0,2 1,1 0,1 0,2 -1,0 -1,1 -1,1 -4,-1 0,1 0,1 0,1 0,1 -1,0 -2,2 -1,-1 0,-1 -1,0 0,-3 0,-1 1,-1 0,-1 1,-1 0,-1 2,-2 0,-1 -1,0 -1,1 -1,0 -1,1 0,-1 0,1 0,1 -1,2 -1,1 -1,-2 0,1 -1,0 1,2 0,2 -1,1 -2,0 0,1 0,1 -1,0 -1,1 0,-1 0,-1 0,-2 0,-2 0,1 -1,-1 0,1 0,2 -1,1 0,1 -1,-1 1,2 0,2 -1,-1 -1,1 1,1 -1,-1 -1,1 -1,0 0,-1 -2,1 0,-1 0,-1 1,-1 0,-1 -1,0 0,-1 1,-1 0,-1 0,-1 -1,0 -1,1 0,1 -1,2 0,-1 -1,0 -1,2 0,-1 0,-1 -1,-2 -2,-1 0,2 0,-1 -2,1 -1,-1 0,-1 1,0 1,-1 0,-1 -1,0 2,0 -1,-1 -1,0 -1,1 -1,0 2,-2 1,-1 1,0 1,-2 3,0 1,0 1,1 2,-1 0,-1 1,-1 1,-2 1,1 1,-1 1,0 2,0 0,-1 0,-1 2,-1 1,-1 2,-1 2,-2 1,-1 1,-1 6,1 2,3 1,0 1,-1 -1,0 0,-1 2,2 0,-1 1,0 -2,-1 0,-2 2,0 0,-1 2,0 1,2 0,1 1,1 1,3 -3,1 -2,0 0,1 -1,0 1,1 -1,1 1,1 m -39,54 0,-2 1,0 2,-2 1,2 0,1 0,1 -1,0 0,1 0,1 0,-1 1,1 2,-2 0,-2 1,0 3,0 4,2 2,5 3,8 1,2 1,3 0,1 -4,2 -4,0 -6,3 -1,0 -7,2 -1,1 -2,0 -3,1 -1,1 0,1 -1,2 -1,1 0,1 -4,1 -1,0 -1,-1 0,-1 -1,1 1,0 -1,2 0,1 -3,2 0,1 -1,3 0,1 -3,3 -2,1 -4,-1 -1,0 0,-1 1,0 -2,-1 -3,3 -4,0 -2,2 -2,-1 0,-1 -1,-3 1,-6 0,-1 0,-2 -5,-9 1,1 1,-1 -1,0 -4,-2 -1,-1 1,0 1,-1 0,-1 2,-1 0,-1 1,0 1,-1 -1,0 2,-1 0,-1 1,0 0,-1 3,0 0,-1 0,-1 1,-1 1,-1 0,-1 0,-1 1,0 3,0 0,-1 0,-1 -1,0 0,-2 1,-1 1,0 2,-3 1,0 0,-1 3,-1 1,-1 0,-1 0,-1 2,0 0,-1 1,0 0,-1 -2,-2 1,-1 0,-2 0,-1 0,-1 1,-2 0,-1 -1,-1 1,-1 0,-1 8,2 5,2 5,0 2,1 0,1 0,1 3,5 3,2 -1,1 -1,1 0,1 0,1 m 73,310 -23,-3 -4,0 -8,-2 -4,0 -1,0 -3,-1 -8,-1 -29,-6 -97,-28 -36,-15 0,-2 -2,0 1,-1 0,-5 1,-3 -2,-1 1,-2 -1,-2 0,-1 2,-1 1,-3 -2,-2 -1,1 -1,1 -1,0 -1,-1 -3,1 -2,-3 -1,-2 -1,0 -1,1 -1,0 0,-1 0,-1 -1,0 0,1 -1,1 -1,-1 -1,-1 -2,1 0,-2 -1,1 0,-1 0,-1 0,-1 1,-1 0,-1 1,-1 -1,-1 1,-1 0,-1 2,-3 -1,-1 1,-2 -1,-1 -2,-1 -2,-5 0,-1 0,-3 -1,0 1,-2 -1,-2 1,0 -1,-3 -1,0 -1,0 -1,-2 -2,0 -1,-2 2,0 -1,-2 1,-1 1,0 0,-2 1,0 -1,-3 -1,0 1,-1 0,-1 0,-1 0,-1 0,-3 1,0 1,-1 2,-1 0,-1 -2,-1 1,-2 -1,0 0,-2 -1,0 1,-1 1,0 1,0 1,-2 1,0 0,-2 0,-1 -1,-1 1,0 0,-1 -1,0 -1,-1 1,0 2,0 0,-1 -2,-2 0,-2 -1,-2 0,-1 0,-1 0,-1 1,-1 0,-1 0,-1 1,-1 -1,-2 -1,-2 -1,1 -2,0 -1,-1 1,-1 1,-1 0,-1 -1,-1 0,-3 0,-1 -1,-2 -2,-1 1,-1 1,0 1,-2 2,0 0,-1 -1,-1 0,-1 1,0 0,-1 -1,0 -1,-1 1,0 0,-1 2,0 1,-1 2,0 2,-2 1,0 1,-3 -1,-3 1,-1 2,0 0,-1 -1,0 0,-1 0,-1 -3,1 -1,-1 1,0 0,-1 -1,-1 -3,0 -1,-1 1,-2 -1,0 1,-1 1,0 2,-2 -2,-2 1,-1 1,-1 1,1 1,-2 1,0 -1,-2 1,-1 0,-1 1,-1 0,-3 0,-1 0,-1 2,1 -16,-10 1,-2 1,-1 0,-1 1,0 1,-1 1,-1 0,-1 1,-2 1,-2 -1,-1 1,-2 14,-19 2,2 1,2 0,1 1,1 0,2 1,0 0,1 0,-4 1,0 -1,-2 3,1 0,1 1,1 1,0 1,1 -1,2 0,1 0,1 1,1 1,-1 -1,-1 1,-5 2,-1 1,1 3,-2 2,0 1,0 1,0 1,0 2,1 2,0 1,-1 1,-1 1,-1 3,2 1,0 -1,0 0,-1 -1,0 1,-1 2,0 -1,0 1,1 -1,0 1,1 1,0 2,0 2,-1 2,0 0,-1 1,1 0,1 -1,1 1,0 0,-1 1,1 1,-2 1,0 0,-1 1,0 1,0 0,1 1,-1 1,0 -1,1 1,0 0,-1 0,1 1,-1 0,1 0,1 -1,0 1,0 1,0 -1,1 1,-1 0,1 1,0 -1,0 1,0 1,-1 1,0 0,-1 1,0 0,1 -1,1 1,1 0,1 0,-1 -1,1 0,1 -9,1 -1,1 -2,-1 -1,2 -1,0 0,-1 0,-1 -2,0 -2,-1 -1,1 0,-1 -2,0 -1,0 0,1 -1,0 0,1 -2,0 -1,1 1,-1 -1,0 0,1 -1,0 -1,1 1,-2 -1,-1 -1,1 -2,1 1,1 -1,0 -1,2 0,-1 1,-1 -1,0 -1,-1 -1,1 0,1 -1,0 0,1 -1,0 0,-1 -1,0 0,1 -1,1 2,3 0,1 1,-1 -2,-1 0,-1 1,1 0,-1 -1,-1 0,-1 1,0 0,2 2,1 0,-1 1,2 0,-1 1,1 1,0 1,0 -1,-2 0,1 0,-1 -1,1 -1,0 1,-1 0,-1 1,1 -1,-2 0,-1 1,0 0,-1 3,1 2,-2 -1,1 0,1 -1,0 1,0 1,-1 0,1 1,-1 -1,1 0,1 1,-2 1,1 1,0 1,-1 -1,0 1,0 2,-1 -1,0 0,-1 -1,0 1,0 1,1 0,-1 1,0 1,1 -1,1 0,1 -1,0 1,0 1,-1 0,-1 1,1 0,-1 0,2 0,-2 1,0 -1,2 -3,1 3,-1 1,-2 1,-2 0,2 -2,3 2,-3 0,-2 1,1 -1,0 0,2 -2,3 1,-1 1,-2 1,-1 -1,2 -2,2 1,-1 1,0 -1,1 -1,1 0,1 -1,0 -1,1 1,1 0,-1 1,-2 2,-1 2,-1 -1,0 4,-2 1,-1 6,0 4,1 2,-1 0,1 -1,0 0,1 -2,-1 -1,2 0,1 1,1 0,-1 2,0 2,-1 2,-1 1,-1 1,-2 2,0 4,1 -2,-3 0,-2 1,1 0,-1 -1,0 1,-3 2,-1 1,6 1,3 -1,6 0,1 0,1 -1,3 1,6 1,3 1,1 1,2 1,0 0,-1 0,-1 1,2 1,-1 1,0 -2,-1 -2,0 1,-1 1,-1 0,1 1,0 1,1 -1,-2 1,0 -1,-1 0,-1 1,-1 1,1 2,0 -1,2 1,-1 0,-1 1,1 -1,-1 -2,-2 0,-1 1,0 0,1 1,0 1,-1 1,-1 2,0 0,-1 -1,1 0,-1 -1,0 -1,0 -1,0 1,-1 2,-1 -1,1 1,1 -1,-1 2,0 1,0 0,2 0,-1 1,1 0,-1 -1,-1 1,0 -2,0 1,0 0,-1 2,1 -1,0 1,1 -1,1 -1,0 1,1 -1,0 0,1 -1,0 1,1 -2,1 0,1 1,2 1,-1 1,1 -1,1 -1,0 -1,1 -2,1 0,1 -1,0 0,1 1,1 1,1 1,-1 0,1 0,-1 1,1 1,1 2,0 0,-1 1,-1 2,1 0,-2 1,-1 1,-2 2,-1 0,1 1,0 3,2 2,0 3,2 3,4 1,3 -12,29 47,69 15,4 8,15 62,22 -8,86 m -25,-268 -1,0 0,-1 0,-1 -2,-1 0,-3 1,0 3,1 -1,5 m 0,2 -2,-1 1,0 0,-1 1,0 0,1 0,1 m -1,3 -1,-2 1,1 0,1 m -61,37 1,-1 2,-1 8,1 10,1 2,0 3,2 3,1 5,3 0,-1 1,0 2,0 -1,-1 2,2 4,0 -1,-1 0,-1 -3,-1 0,-3 -2,0 0,-1 -1,0 -1,-1 -1,0 -1,-2 -2,-2 -1,1 -2,-2 -2,0 -1,0 -2,0 -5,-1 -2,0 -3,-1 0,-1 -1,0 -4,-1 -3,-1 -1,0 -1,-1 0,-1 0,-1 -1,-3 -1,-2 0,-2 5,-2 7,1 0,-1 4,-1 1,1 2,0 -2,-2 3,0 1,0 1,1 3,0 -3,0 -1,-1 1,0 -1,-1 -12,0 0,-1 1,-1 -1,0 0,-1 -1,1 1,1 -1,1 -1,-1 -1,0 -2,-1 0,-1 0,-1 2,-1 2,1 -1,-1 2,0 -2,-1 2,-1 -2,0 -1,0 -1,1 -1,0 0,-1 0,-1 -2,-1 -1,1 0,1 -1,-2 1,-3 1,-2 2,-2 1,-1 1,-1 1,0 3,-1 1,-1 -2,-2 0,-1 1,-2 3,-2 2,0 4,-4 3,0 3,-1 13,-3 2,2 1,3 0,1 -2,4 0,1 0,2 -2,0 -1,0 0,2 -2,1 0,1 1,0 2,-1 1,0 1,0 2,0 -2,1 2,-1 1,-1 -1,-1 2,-3 2,-1 3,2 1,1 1,1 5,5 0,1 -1,2 -2,1 -1,1 -2,1 0,1 1,0 0,1 -1,1 0,1 1,-2 2,-1 -1,0 0,-1 2,-1 1,0 -1,1 -1,0 1,0 -1,1 1,0 1,2 0,-3 1,0 0,-1 1,0 0,-1 1,-1 1,1 -1,1 2,2 -1,-2 1,-1 2,3 -11,50 -17,-4 0,2 -1,0 -1,1 -1,-1 0,-3 -28,-8 -1,2 2,6 -2,-1 -2,-3 1,-5 m 78,-114 -1,-1 -1,-1 1,-1 -1,0 1,-2 2,0 -1,5 m -7,29 -1,-1 -1,1 1,0 -2,0 0,2 -7,3 -5,0 -5,-1 -2,-1 -2,-4 0,-2 5,-2 2,1 0,-1 3,-2 4,1 2,1 1,-1 1,0 -1,-1 2,0 2,-1 -1,-1 1,0 0,-1 -1,0 -1,1 -2,0 1,1 -1,0 0,-1 -1,1 -3,-1 0,-1 1,0 0,-1 -1,0 0,1 -1,0 0,-1 1,-1 -1,0 0,1 0,1 -1,1 -1,0 -1,-1 0,1 -1,0 -3,-1 0,-1 2,-1 -2,0 2,-3 3,-1 -1,-1 -2,1 0,-2 -1,0 0,1 0,1 -2,2 -1,-2 -1,1 1,0 0,2 -1,2 -3,1 -1,0 0,-2 0,-1 0,-1 0,1 -1,0 -1,-1 0,1 0,1 -1,1 -3,-2 2,-1 -2,0 1,-2 -1,0 -1,1 -4,-3 -1,-2 1,-1 1,0 1,-1 1,-1 7,1 2,-1 2,0 0,-1 2,0 -4,0 -3,0 -3,-1 -2,-1 2,-2 1,-1 4,1 2,-1 1,1 3,0 1,0 -7,-1 -2,-2 0,-1 1,0 0,-1 1,0 -1,-1 0,-1 1,-1 2,0 7,3 -2,-3 -3,-1 0,-1 1,0 -1,-1 3,-2 3,1 0,-1 1,0 0,1 1,0 2,2 -1,1 0,1 0,2 0,1 4,-1 1,1 1,0 0,1 1,1 0,2 0,1 1,1 2,2 -2,1 -1,1 4,1 -1,0 0,1 0,1 1,0 -1,1 0,1 0,1 0,1 4,0 1,1 2,1 -4,14 m 14,-65 -1,1 -3,0 0,1 1,1 2,0 0,1 0,1 0,1 -1,1 0,1 0,1 -4,1 -2,0 -4,1 -1,-1 -1,-1 0,-1 -1,-1 0,-1 -1,-1 0,-1 1,0 0,-2 0,-1 0,-1 1,-3 1,1 1,-1 2,-1 2,0 1,0 0,1 3,-1 1,1 2,-1 2,1 -1,3 m 3,-18 1,3 -2,7 -2,1 -1,-1 -1,-2 -1,0 0,1 -1,0 0,1 -2,-1 0,-1 -1,-1 -4,1 -1,-2 1,0 0,-1 3,-1 2,-1 2,0 1,0 1,-1 1,-1 2,-1 2,0"
                    }
                }
            }
        }
    );

    return Mapael;

}));
